@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.c,v 1.4 2003/05/11 18:17:54 fuyu Exp $
+ * $Id: bbs_thread_ui.c,v 1.6 2003/05/15 19:55:43 fuyu Exp $
  */
 
 #include "config.h"
@@ -188,6 +188,20 @@ enum {
 
 
 static GtkItemFactory *link_popup_menu_item_factory = NULL;
+static GQuark bbs_thread_gui_info_id;
+
+
+void
+prepare_thread_ui_initialization(OchushaApplication *application)
+{
+  if (pthread_mutex_init(&bbs_thread_ui_lock, NULL) != 0)
+    {
+      fprintf(stderr, "Couldn't init a mutex.\n");
+      abort();
+    }
+
+  bbs_thread_gui_info_id = g_quark_from_static_string("BBSThreadUI::GUIInfo");
+}
 
 
 void
@@ -262,12 +276,6 @@ initialize_thread_ui(OchushaApplication *application)
 #endif
   g_return_if_fail(tag_table == NULL);
 
-  if (pthread_mutex_init(&bbs_thread_ui_lock, NULL) != 0)
-    {
-      fprintf(stderr, "Couldn't init a mutex.\n");
-      abort();
-    }
-
   link_field_id = gtk_statusbar_get_context_id(application->statusbar,
 					       "response-link-field");
   popup_response_window = gtk_window_new(GTK_WINDOW_POPUP);
@@ -332,11 +340,12 @@ ensure_bbs_thread_info(OchushaBBSThread *thread)
   BBSThreadGUIInfo *info;
   BBS_THREAD_UI_LOCK
     {
-      info = (BBSThreadGUIInfo *)thread->user_data;
+      info = g_object_get_qdata(G_OBJECT(thread), bbs_thread_gui_info_id);
       if (info == NULL)
 	{
-	  info = (BBSThreadGUIInfo *)calloc(1, sizeof(BBSThreadGUIInfo));
-	  thread->user_data = info;
+	  info = g_new0(BBSThreadGUIInfo, 1);
+	  g_object_set_qdata_full(G_OBJECT(thread), bbs_thread_gui_info_id,
+				  info, (GDestroyNotify)g_free);
 	}
     }
   BBS_THREAD_UI_UNLOCK;
@@ -599,7 +608,7 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
   BBSThreadJobArgs *job_args = (BBSThreadJobArgs *)args;
   OchushaApplication *application = job_args->application;
   OchushaBBSThread *thread = job_args->thread;
-  BBSThreadGUIInfo *info = (BBSThreadGUIInfo *)thread->user_data;
+  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
   OchushaAsyncBuffer *buffer = job_args->buffer;
   gchar default_text_buffer[DEFAULT_BUFFER_SIZE];
   RendererContext context =
@@ -751,7 +760,7 @@ start_thread_rendering(OchushaBBSThread *thread, const gchar *title,
   if (context->last_read_number == 0)
     {
       /* ɤॹξ祿ȥߤˤΤǤ˥ޡդ롣*/
-      BBSThreadGUIInfo *info = (BBSThreadGUIInfo *)context->thread->user_data;
+      BBSThreadGUIInfo *info = ensure_bbs_thread_info(context->thread);
       info->last_read = bbs_thread_view_create_mark(view);
       info->next_mark = info->last_read;
       bbs_thread_view_scroll_to_mark(view, info->last_read);
@@ -783,7 +792,7 @@ render_response(OchushaBBSThread *thread, gint number,
 {
   RendererContext *context = (RendererContext *)user_data;
   BBSThreadView *view = context->view;
-  BBSThreadGUIInfo *info = (BBSThreadGUIInfo *)thread->user_data;
+  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
   char buffer[13];
   const char *cur_pos;
 #if DEBUG_GUI_MOST
@@ -885,7 +894,7 @@ end_thread_rendering(OchushaBBSThread *thread, gboolean finished,
 {
   RendererContext *context = (RendererContext *)user_data;
   BBSThreadView *view = context->view;
-  BBSThreadGUIInfo *info = (BBSThreadGUIInfo *)context->thread->user_data;
+  BBSThreadGUIInfo *info = ensure_bbs_thread_info(context->thread);
 
   if (!finished)
     {
@@ -1094,7 +1103,7 @@ prepare_buffer_for_popup(OchushaApplication *application,
 {
   unsigned int from = 0;
   unsigned int to = 0;
-  BBSThreadGUIInfo *info = (BBSThreadGUIInfo *)thread->user_data;
+  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
 
   if (application->maximum_number_of_popup_responses < 1)
     return FALSE;
@@ -1488,7 +1497,7 @@ void
 refresh_thread(OchushaApplication *application, GtkWidget *widget,
 	       OchushaBBSThread *thread, IconLabel *tab_label)
 {
-  BBSThreadGUIInfo *info = (BBSThreadGUIInfo *)thread->user_data;
+  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
   BBSThreadView *view = BBS_THREAD_VIEW(widget);
   WorkerJob *job = NULL;
   BBSThreadJobArgs *job_args = NULL;
@@ -1604,9 +1613,9 @@ void
 jump_to_new_comer_response(GtkWidget *widget, OchushaBBSThread *thread)
 {
   BBSThreadView *view;
-  g_return_if_fail(widget != NULL && thread != NULL
-		   && thread->user_data != NULL);
+  g_return_if_fail(widget != NULL && OCHUSHA_IS_BBS_THREAD(thread));
+
   view = BBS_THREAD_VIEW(widget);
   bbs_thread_view_scroll_to_mark(view,
-			((BBSThreadGUIInfo *)thread->user_data)->last_read);
+				 ensure_bbs_thread_info(thread)->last_read);
 }
