@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.h,v 1.1.1.1 2003/05/10 16:34:24 fuyu Exp $
+ * $Id: bbs_thread_ui.h,v 1.3 2003/05/15 19:55:43 fuyu Exp $
  */
 
 #ifndef _BBSTHREAD_UI_H_
@@ -38,13 +38,26 @@
 extern "C" {
 #endif
 
+
+typedef enum
+{
+  BBS_THREAD_FAVORITE	= 1 << 0,
+  BBS_THREAD_HIDDEN	= 1 << 1,
+  BBS_THREAD_NEW	= 1 << 2,
+} ThreadlistEntryFlags;
+
+
 /*
  * åɤGUIϢ
  */
 /* ThreadlistInfoΥ֥饹Ū˻Ȥ */
 typedef struct _BBSThreadGUIInfo
 {
-  ThreadlistInfo info;
+  /* ߤΥ */
+  int view_rank;
+
+  /* ɽ(ThreadlistEntryFlagsORȤä) */
+  int view_flags;
 
   /* ǽο쥹Ƭդޡ */
   GtkTextMark *last_read;
@@ -76,6 +89,7 @@ typedef enum
 #endif
 
 
+void prepare_thread_ui_initialization(OchushaApplication *application);
 void initialize_thread_ui(OchushaApplication *application);
 BBSThreadGUIInfo * ensure_bbs_thread_info(OchushaBBSThread *thread);
 GtkWidget *open_bbs_thread(OchushaApplication *application,
