@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.c,v 1.1.1.1 2003/05/10 16:34:25 fuyu Exp $
+ * $Id: bbs_thread_view.c,v 1.5 2003/05/19 04:56:58 fuyu Exp $
  */
 
 #include "config.h"
@@ -45,6 +45,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 
 static GtkTextTag *get_alt_link_tag(BBSThreadView *view);
@@ -201,7 +202,6 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
   gtk_text_tag_table_add(default_tag_table, tag);
   link_tags[1] = tag;
 
-#if 1	/* ɤäɤʡġġ */
   tag = gtk_text_tag_new("response_paragraph");
   g_object_set(G_OBJECT(tag),
 	       "indent", 0,
@@ -209,13 +209,6 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
 	       "left-margin-set", TRUE,
 	       NULL);
   gtk_text_tag_table_add(default_tag_table, tag);
-#else
-  tag = gtk_text_tag_new("response_paragraph");
-  g_object_set(G_OBJECT(tag),
-	       "indent", 30,
-	       NULL);
-  gtk_text_tag_table_add(default_tag_table, tag);
-#endif
 
   tag = gtk_text_tag_new("bold");
   g_object_set(G_OBJECT(tag),
@@ -228,6 +221,13 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
 	       "style", PANGO_STYLE_ITALIC,
 	       NULL);
   gtk_text_tag_table_add(default_tag_table, tag);
+
+  tag = gtk_text_tag_new("match-string");
+  g_object_set(G_OBJECT(tag),
+	       "foreground", "red",
+	       "background", "#ffffd0",
+	       "weight", PANGO_WEIGHT_BOLD,
+	       NULL);
 			 
   bbs_thread_view_signals[LINK_MOUSE_OVER_SIGNAL] =
     g_signal_new("link_mouse_over",
@@ -312,22 +312,9 @@ bbs_thread_view_init(BBSThreadView *view)
   GtkWidget *widget;
   GtkTextBuffer *buffer = gtk_text_buffer_new(default_tag_table);
 
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "bbs_thread_view_init() <pre gtk_text_view_new_with_buffer()> text_buffer->ref_count=%d\n",
-	  G_OBJECT(buffer)->ref_count);
-#endif
-
   widget = gtk_text_view_new_with_buffer(buffer);
 
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "bbs_thread_view_init() <post gtk_text_view_new_with_buffer()> text_buffer->ref_count=%d\n",
-	  G_OBJECT(buffer)->ref_count);
-#endif
-
   view->text_buffer = buffer;
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "bbs_thread_view_init: text_view=0x%x\n", (int)widget);
-#endif
   view->text_view = GTK_TEXT_VIEW(widget);
   gtk_text_view_set_wrap_mode(view->text_view, GTK_WRAP_CHAR);
   gtk_text_view_set_editable(view->text_view, FALSE);
@@ -355,7 +342,6 @@ bbs_thread_view_init(BBSThreadView *view)
   g_signal_connect(G_OBJECT(widget), "button_release_event",
 		   G_CALLBACK(button_release_event_cb), view);
 
-  /*  view->text_buffer = gtk_text_view_get_buffer(view->text_view); */
   gtk_text_buffer_get_start_iter(view->text_buffer, &view->iter);
 
   view->tags = NULL;
@@ -363,51 +349,19 @@ bbs_thread_view_init(BBSThreadView *view)
   view->last_offset = -1;
   view->thread = NULL;
   view->link_table = g_hash_table_new_full(NULL, NULL, NULL, g_free);
+
+  view->insert_mark = gtk_text_buffer_get_mark(view->text_buffer, "insert");
+  view->selection_bound_mark = gtk_text_buffer_get_mark(view->text_buffer,
+							"selection_bound");
 }
 
 
 static void
 bbs_thread_view_finalize(GObject *object)
 {
+#if 0
   BBSThreadView *view = BBS_THREAD_VIEW(object);
-#if DEBUG_WIDGET
-  fprintf(stderr, "bbs_thread_view_finalize: object=0x%x\n", (int)object);
-  fprintf(stderr, "* view->buffer->ref_count=%d\n",
-	  G_OBJECT(view->text_buffer)->ref_count);
 #endif
-  if (view->text_buffer != NULL)
-    {
-      g_object_unref(G_OBJECT(view->text_buffer));
-      view->text_buffer = NULL;
-    }
-  if (view->tags != NULL)
-    g_slist_free(view->tags);
-
-  if (view->link_table != NULL)
-    {
-      g_hash_table_destroy(view->link_table);
-      view->link_table = NULL;
-    }
-
-  if (view->ibeam_cursor != NULL)
-    {
-#if DEBUG_WIDGET_MOST
-      fprintf(stderr, "ibeam_cursor->ref_count=%d\n",
-	      view->ibeam_cursor->ref_count);
-#endif
-      gdk_cursor_unref(view->ibeam_cursor);
-      view->ibeam_cursor = NULL;
-    }
-
-  if (view->hand_cursor != NULL)
-    {
-#if DEBUG_WIDGET_MOST
-      fprintf(stderr, "hand_cursor->ref_count=%d\n",
-	      view->hand_cursor->ref_count);
-#endif
-      gdk_cursor_unref(view->hand_cursor);
-      view->hand_cursor = NULL;
-    }
 
   if (G_OBJECT_CLASS(parent_class)->finalize)
     (*G_OBJECT_CLASS(parent_class)->finalize)(object);
@@ -417,11 +371,49 @@ bbs_thread_view_finalize(GObject *object)
 static void
 bbs_thread_view_destroy(GtkObject *object)
 {
-#if DEBUG_WIDGET
   BBSThreadView *view = BBS_THREAD_VIEW(object);
-  fprintf(stderr, "bbs_thread_view_destroy: object=0x%x\n", (int)object);
-  fprintf(stderr, "* view->buffer->ref_count=%d\n",
-	  G_OBJECT(view->text_buffer)->ref_count);
+
+  if (view->text_buffer != NULL)
+    {
+      g_object_unref(G_OBJECT(view->text_buffer));
+      view->text_buffer = NULL;
+    }
+  if (view->tags != NULL)
+    {
+      g_slist_free(view->tags);
+      view->tags = NULL;
+    }
+
+  if (view->link_table != NULL)
+    {
+      g_hash_table_destroy(view->link_table);
+      view->link_table = NULL;
+    }
+
+  if (view->ibeam_cursor != NULL)
+    {
+      gdk_cursor_unref(view->ibeam_cursor);
+      view->ibeam_cursor = NULL;
+    }
+
+  if (view->hand_cursor != NULL)
+    {
+      gdk_cursor_unref(view->hand_cursor);
+      view->hand_cursor = NULL;
+    }
+
+  if (view->search_key != NULL)
+    {
+      g_free(view->search_key);
+      view->search_key = NULL;
+    }
+
+#if ENABLE_REGEXP
+  if (view->regexp_available)
+    {
+      regfree(&view->regexp);
+      view->regexp_available = FALSE;
+    }
 #endif
 
   if (GTK_OBJECT_CLASS(parent_class)->destroy)
@@ -553,14 +545,6 @@ emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event)
   if (link == NULL)
     return;
 
-#if DEBUG_WIDGET_MOST
-  {
-    gchar *native_link = convert_string(utf8_to_native, link, -1);
-    fprintf(stderr, "link_mouse_over: %s\n", native_link);
-    free(native_link);
-  }
-#endif
-
   gdk_window_set_cursor(gtk_text_view_get_window(view->text_view,
 						 GTK_TEXT_WINDOW_TEXT),
 			view->hand_cursor);
@@ -592,14 +576,6 @@ emit_link_mouse_out(BBSThreadView *view, GdkEventMotion *event)
   if (link == NULL)
     return;
 
-#if DEBUG_WIDGET_MOST
-  {
-    gchar *native_link = convert_string(utf8_to_native, link, -1);
-    fprintf(stderr, "link_mouse_out: %s\n", native_link);
-    free(native_link);
-  }
-#endif
-
   gdk_window_set_cursor(gtk_text_view_get_window(view->text_view,
 						 GTK_TEXT_WINDOW_TEXT),
 			view->ibeam_cursor);
@@ -624,11 +600,6 @@ button_press_event_cb(GtkTextView *text_view, GdkEventButton *event,
 			   || event->type == GDK_2BUTTON_PRESS
 			   || event->type == GDK_3BUTTON_PRESS), FALSE);
 
-#if DEBUG_WIDGET_MOST
-  if (bbs_thread_view->link_table == NULL)
-    fprintf(stderr, "XXX button_press: ?");
-#endif
-
   if (text_view_get_iter_at_link_tag_from_event(text_view, &iter,
 						(GdkEventAny *)event))
     {
@@ -639,13 +610,6 @@ button_press_event_cb(GtkTextView *text_view, GdkEventButton *event,
 				       (gconstpointer)start_offset)
 	: NULL;
 
-#if DEBUG_WIDGET_MOST
-      {
-	gchar *native_link = convert_string(utf8_to_native, link, -1);
-	fprintf(stderr, "link_mouse_press: %s\n", native_link);
-	free(native_link);
-      }
-#endif
       g_signal_emit(G_OBJECT(bbs_thread_view),
 		    bbs_thread_view_signals[LINK_MOUSE_PRESS_SIGNAL],
 		    0,
@@ -669,11 +633,6 @@ button_release_event_cb(GtkTextView *text_view, GdkEventButton *event,
   g_return_val_if_fail(GTK_IS_TEXT_VIEW(text_view)
 		       && event->type == GDK_BUTTON_RELEASE, FALSE);
 
-#if DEBUG_WIDGET_MOST
-  if (bbs_thread_view->link_table == NULL)
-    fprintf(stderr, "XXX button_press: ?");
-#endif
-
   if (text_view_get_iter_at_link_tag_from_event(text_view, &iter,
 						(GdkEventAny *)event))
     {
@@ -684,13 +643,6 @@ button_release_event_cb(GtkTextView *text_view, GdkEventButton *event,
 				       (gconstpointer)start_offset)
 	: NULL;
 
-#if DEBUG_WIDGET_MOST
-      {
-	gchar *native_link = convert_string(utf8_to_native, link, -1);
-	fprintf(stderr, "link_mouse_release: %s\n", native_link);
-	free(native_link);
-      }
-#endif
       g_signal_emit(G_OBJECT(bbs_thread_view),
 		    bbs_thread_view_signals[LINK_MOUSE_RELEASE_SIGNAL],
 		    0,
@@ -796,10 +748,6 @@ bbs_thread_view_create_mark(BBSThreadView *view)
 
   mark = gtk_text_buffer_create_mark(view->text_buffer, NULL, &view->iter,
 				     TRUE); 
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "bbs_thread_view_create_mark: text_view=0x%x mark=0x%x\n",
-	  (int)view->text_view, (int)mark);
-#endif
   return mark;
 }
 
@@ -818,11 +766,6 @@ bbs_thread_view_delete_mark(BBSThreadView *view, GtkTextMark *mark)
   if (buffer == NULL)
     return;
 
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "bbs_thread_view_delete_mark: text_view=0x%x mark=0x%x\n",
-	  (int)view->text_view, (int)mark);
-#endif
-
   gtk_text_buffer_delete_mark(buffer, mark);
 }
 
@@ -863,14 +806,6 @@ bbs_thread_view_append_text(BBSThreadView *view, const gchar *text, int len)
   args.buffer = view->text_buffer;
   start_offset = gtk_text_iter_get_offset(&view->iter);
 
-#if DEBUG_WIDGET_MOST
-  {
-    gchar *native_text = convert_string(utf8_to_native, text, len);
-    fprintf(stderr, "append_text: %s\n", native_text);
-    free(native_text);
-  }
-#endif
-
   gtk_text_buffer_insert(view->text_buffer, &view->iter, text, len);
   gtk_text_buffer_get_iter_at_offset(view->text_buffer, &start, start_offset);
   g_slist_foreach(view->tags, apply_tag, &args);
@@ -938,12 +873,7 @@ bbs_thread_view_push_tag_by_name(BBSThreadView *view, const gchar *name)
   tag = gtk_text_tag_table_lookup(default_tag_table, name);
 
   if (tag != NULL)
-    {
-#if DEBUG_WIDGET_MOST
-      fprintf(stderr, "tag named \"%s\" found.\n", name);
-#endif
-      bbs_thread_view_push_tag(view, tag);
-    }
+    bbs_thread_view_push_tag(view, tag);
 }
 
 
@@ -957,10 +887,6 @@ bbs_thread_view_push_tag(BBSThreadView *view, GtkTextTag *tag)
 
   g_return_if_fail(GTK_IS_TEXT_TAG(tag));
 
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "tag(0x%x) pushed.\n", (int)tag);
-#endif
-
   view->tags = g_slist_prepend(view->tags, tag);
 }
 
@@ -976,24 +902,12 @@ bbs_thread_view_pop_tag(BBSThreadView *view)
   head_entry = view->tags;
 
   if (head_entry == NULL)
-    {
-#if DEBUG_WIDGET_MOST
-      fprintf(stderr, "There's no tags.\n");
-#endif
-      return NULL;
-    }
+    return NULL;
 
   tag = GTK_TEXT_TAG(head_entry->data);
   view->tags = g_slist_remove_link(view->tags, head_entry);
   g_slist_free(head_entry);
 
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "tag(0x%x) popped.\n", (int)tag);
-
-  if (view->tags == NULL)
-    fprintf(stderr, "view->tags = NULL\n");
-#endif
-
   return tag;
 }
 
@@ -1022,10 +936,6 @@ bbs_thread_view_scroll_to(BBSThreadView *view, gint to_where)
 {
   g_return_if_fail(IS_BBS_THREAD_VIEW(view));
 
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "bbs_thread_view_scroll_to(view, %d)\n", to_where);
-#endif
-
   if (to_where == SCROLL_TO_BUFFER_START)
     bbs_thread_view_scroll_to_start(view);
   else
@@ -1047,11 +957,8 @@ bbs_thread_view_scroll_to_start(BBSThreadView *view)
 
   gtk_text_buffer_get_start_iter(buffer, &iter);
   gtk_text_buffer_place_cursor(buffer, &iter);
-  result = gtk_text_view_scroll_to_iter(view->text_view, &iter, 0.0, FALSE, 0.0, 0.0);
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "scroll_to_start: result=%d, offset=%d\n",
-	  result, gtk_text_iter_get_offset(&iter));
-#endif
+  result = gtk_text_view_scroll_to_iter(view->text_view,
+					&iter, 0.0, FALSE, 0.0, 0.0);
 }
 
 
@@ -1069,11 +976,8 @@ bbs_thread_view_scroll_to_end(BBSThreadView *view)
 
   gtk_text_buffer_get_end_iter(buffer, &iter);
   gtk_text_buffer_place_cursor(buffer, &iter);
-  result = gtk_text_view_scroll_to_iter(view->text_view, &iter, 0.0, FALSE, 0.0, 0.0);
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "scroll_to_end: result=%d, offset=%d\n",
-	  result, gtk_text_iter_get_offset(&iter));
-#endif
+  result = gtk_text_view_scroll_to_iter(view->text_view,
+					&iter, 0.0, FALSE, 0.0, 0.0);
 }
 
 
@@ -1094,11 +998,9 @@ bbs_thread_view_scroll_to_offset(BBSThreadView *view, gint offset)
 
   gtk_text_buffer_get_iter_at_offset(view->text_buffer, &iter, offset);
   gtk_text_buffer_place_cursor(view->text_buffer, &iter);
-  result = gtk_text_view_scroll_to_iter(view->text_view, &iter, 0.0, FALSE, 0.0, 0.0);
-#if DEBUG_WIDGET
-  fprintf(stderr, "scroll_to_offset: result=%d, offset=%d\n",
-	  result, gtk_text_iter_get_offset(&iter));
-#endif
+  result = gtk_text_view_scroll_to_iter(view->text_view,
+					&iter, 0.0, FALSE, 0.0, 0.0);
+
   return previous_offset;
 }
 
@@ -1118,20 +1020,11 @@ bbs_thread_view_scroll_to_mark(BBSThreadView *view, GtkTextMark *mark)
   gtk_text_buffer_get_iter_at_mark(buffer, &iter, mark);
   gtk_text_buffer_place_cursor(buffer, &iter);
 
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "bbs_thread_view_scroll_to_mark: text_view=0x%x mark=0x%x\n",
-	  (int)view->text_view, (int)mark);
-#endif
 #if 0	/* XXX: ɤäʡ */
   gtk_text_view_scroll_to_mark(view->text_view, mark, 0.3, FALSE, 0.5, 0.5);
 #else
   gtk_text_view_scroll_to_mark(view->text_view, mark, 0.3, TRUE, 0.5, 0.5);
 #endif
-
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "scroll_to_mark: offset=%d\n",
-	  gtk_text_iter_get_offset(&iter));
-#endif
 }
 
 
@@ -1140,10 +1033,6 @@ bbs_thread_view_place_cursor_onscreen(BBSThreadView *view)
 {
   g_return_if_fail(IS_BBS_THREAD_VIEW(view) && view->text_view != NULL);
 
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "bbs_thread_view_place_cursor_onscreen: text_view=0x%x\n",
-	  (int)view->text_view);
-#endif
   gtk_text_view_place_cursor_onscreen(view->text_view);
 }
 
@@ -1156,3 +1045,423 @@ bbs_thread_view_hide_scrollbar(BBSThreadView *view)
   gtk_scrolled_window_set_policy(&view->text_container,
 				 GTK_POLICY_NEVER, GTK_POLICY_NEVER);
 }
+
+
+/* Ϣ */
+static void
+bbs_thread_view_scroll_to_offsets(BBSThreadView *view,
+				  gint start_offset, gint end_offset)
+{
+  /* ޥåʬ򤷡ʬɽ褦˥ */
+  GtkTextIter iter;
+  gtk_text_buffer_get_iter_at_offset(view->text_buffer, &iter,
+				     start_offset);
+  gtk_text_buffer_move_mark(view->text_buffer, view->insert_mark, &iter);
+  gtk_text_buffer_get_iter_at_offset(view->text_buffer, &iter,
+				     + end_offset);
+  gtk_text_buffer_move_mark(view->text_buffer, view->selection_bound_mark,
+			    &iter);
+  gtk_text_view_scroll_mark_onscreen(view->text_view,
+				     view->insert_mark);
+  gtk_text_view_scroll_mark_onscreen(view->text_view,
+				     view->selection_bound_mark);
+}
+
+
+static gboolean
+bbs_thread_view_find_forward_first_match(BBSThreadView *view,
+					 GtkTextIter *start_iter,
+					 GtkTextIter *end_iter,
+					 gint *match_start_offset_p,
+					 gint *match_end_offset_p)
+{
+  gboolean result = FALSE;
+  gchar *text;
+#if !ENABLE_REGEXP
+  g_return_val_if_fail(!view->use_regexp, FALSE);
+#endif
+  /*
+   * MEMO: gtk_text_iter_get_text()䤽֤ϥԡ֤Ƥ뤳Ȥ
+   *       աХåեΤоݤ˸ȤʤȡХåեݤȤΥԡ
+   *       롣ٹʤΤǤʺ٤ʤȤ̵Ǥ
+   */
+  g_return_val_if_fail(!view->use_regexp || view->regexp_available, FALSE);
+
+  text = gtk_text_iter_get_slice(start_iter, end_iter);
+  g_return_val_if_fail(text != NULL, FALSE);
+
+  if (!view->use_regexp && !view->match_case)
+    {
+      gchar *tmp_text = g_utf8_casefold(text, -1);
+      g_free(text);
+      text = tmp_text;
+    }
+
+  if (view->use_regexp)
+    {
+#if ENABLE_REGEXP
+      regmatch_t match;
+      int regexec_result = regexec(&view->regexp, text, 1, &match, 0);
+
+      if (regexec_result == 0)
+	{
+	  gint offset = gtk_text_iter_get_offset(start_iter);
+	  *match_start_offset_p = g_utf8_strlen(text, match.rm_so) + offset;
+	  *match_end_offset_p
+	    = g_utf8_strlen(text + match.rm_so, match.rm_eo - match.rm_so)
+	    + (*match_start_offset_p);
+	  result = TRUE;
+	}
+#else
+      /* ºݤˤ뤳ȤϤʤ */
+#endif
+    }
+  else
+    {
+      gchar *match_start = strstr(text, view->search_key);
+      if (match_start != NULL)
+	{
+	  gint offset = gtk_text_iter_get_offset(start_iter);
+	  *match_start_offset_p
+	    = g_utf8_strlen(text, match_start - text) + offset;
+	  *match_end_offset_p
+	    = g_utf8_strlen(view->search_key, -1) + (*match_start_offset_p);
+	  result = TRUE;
+	}
+    }
+
+  if (text != NULL)
+    g_free(text);
+
+  return result;
+}
+
+
+static gboolean
+bbs_thread_view_find_backward_first_match(BBSThreadView *view,
+					  GtkTextIter *start_iter,
+					  GtkTextIter *end_iter,
+					  gint *match_start_offset_p,
+					  gint *match_end_offset_p)
+{
+  gboolean result = FALSE;
+  gchar *text;
+#if !ENABLE_REGEXP
+  g_return_val_if_fail(!view->use_regexp, FALSE);
+#endif
+  /*
+   * MEMO: gtk_text_iter_get_text()䤽֤ϥԡ֤Ƥ뤳Ȥ
+   *       աХåեΤоݤ˸ȤʤȡХåեݤȤΥԡ
+   *       롣ٹʤΤǤʺ٤ʤȤ̵Ǥ
+   */
+  text = gtk_text_iter_get_slice(start_iter, end_iter);
+  g_return_val_if_fail(text != NULL, FALSE);
+
+  if (!view->use_regexp && !view->match_case)
+    {
+      gchar *tmp_text = g_utf8_casefold(text, -1);
+      g_free(text);
+      text = tmp_text;
+    }
+
+  if (view->use_regexp)
+    {
+#if ENABLE_REGEXP
+      /*
+       * MEMO: ɽεսޥåˤϤޤˡ̵ʤΤǡ٤
+       *       (ޥåʸ¿)ǰξO(n)regexec()¹Ԥ롣
+       */
+      gboolean found = FALSE;
+      regmatch_t match;
+      gchar *tmp_pos = text;
+      gchar *last_start_pos = NULL;
+      gchar *last_end_pos = NULL;
+
+      while (*tmp_pos != '\0'
+	     && regexec(&view->regexp, tmp_pos, 1, &match, 0) == 0)
+	{
+	  found = TRUE;
+	  last_start_pos = match.rm_so + tmp_pos;
+	  last_end_pos = match.rm_eo + tmp_pos;
+	  tmp_pos += match.rm_eo;
+	}
+
+      if (found)
+	{
+	  gint offset = gtk_text_iter_get_offset(start_iter);
+	  *match_start_offset_p
+	    = g_utf8_strlen(text, last_start_pos - text) + offset;
+	  *match_end_offset_p
+	    = g_utf8_strlen(last_start_pos, last_end_pos - last_start_pos)
+	    + (*match_start_offset_p);
+	  result = TRUE;
+	}
+#else
+      /* ºݤˤ뤳ȤϤʤ */
+#endif
+    }
+  else
+    {
+      gchar *match_start = g_strrstr(text, view->search_key);
+      if (match_start != NULL)
+	{
+	  gint offset = gtk_text_iter_get_offset(start_iter);
+	  *match_start_offset_p
+	    = g_utf8_strlen(text, match_start - text) + offset;
+	  *match_end_offset_p
+	    = g_utf8_strlen(view->search_key, -1) + (*match_start_offset_p);
+	  result = TRUE;
+	}
+    }
+
+  if (text != NULL)
+    g_free(text);
+
+  return result;
+}
+
+
+static gboolean
+bbs_thread_view_do_search_forward(BBSThreadView *view,
+				  gboolean include_current_pos)
+{
+  GtkTextIter start;
+  GtkTextIter end;
+  gint start_offset = 0;
+  gint end_offset = 0;
+
+#if !ENABLE_REGEXP
+  g_return_val_if_fail(!view->use_regexp, FALSE);
+#endif
+
+  if (view->use_regexp)
+    {
+      if (!view->regexp_available)
+	return FALSE;
+    }
+  else
+    {
+      if (view->search_key == NULL)
+	return FALSE;
+    }
+
+  gtk_text_buffer_get_iter_at_offset(view->text_buffer, &start,
+				     view->last_match_offset);
+  if (!include_current_pos)
+    {
+      if (!gtk_text_iter_forward_char(&start))
+	{
+	  if (view->enable_wrap)
+	    view->last_match_offset = 0;
+	  return FALSE;
+	}
+    }
+  gtk_text_buffer_get_end_iter(view->text_buffer, &end);
+
+  if (bbs_thread_view_find_forward_first_match(view, &start, &end,
+					       &start_offset, &end_offset))
+    {
+      bbs_thread_view_scroll_to_offsets(view, start_offset, end_offset);
+      view->last_match_offset = start_offset;
+      return TRUE;
+    }
+  else if (view->enable_wrap)
+    view->last_match_offset = 0;
+
+  return FALSE;
+}
+
+
+static gboolean
+bbs_thread_view_do_search_backward(BBSThreadView *view,
+				   gboolean include_current_pos)
+{
+  GtkTextIter start;
+  GtkTextIter end;
+  gint start_offset = 0;
+  gint end_offset = 0;
+
+#if !ENABLE_REGEXP
+  g_return_val_if_fail(!view->use_regexp, FALSE);
+#endif
+
+  if (view->use_regexp)
+    {
+      if (!view->regexp_available)
+	return FALSE;
+    }
+  else
+    if (view->search_key == NULL)
+      return FALSE;
+
+  gtk_text_buffer_get_start_iter(view->text_buffer, &start);
+  gtk_text_buffer_get_iter_at_offset(view->text_buffer, &end,
+				     view->last_match_offset);
+
+  if (include_current_pos)
+    {
+      if (view->use_regexp)
+	{
+	  /* ʤ */
+	  if (!gtk_text_iter_forward_line(&end))
+	    gtk_text_buffer_get_end_iter(view->text_buffer, &end);
+	}
+      else
+	gtk_text_iter_forward_chars(&end, g_utf8_strlen(view->search_key, -1));
+    }
+  else
+    {
+      if (!gtk_text_iter_backward_char(&end))
+	{
+	  if (view->enable_wrap)
+	    {
+	      gtk_text_buffer_get_end_iter(view->text_buffer, &end);
+	      view->last_match_offset = gtk_text_iter_get_offset(&end);
+	    }
+	  return FALSE;
+	}
+    }
+
+  if (bbs_thread_view_find_backward_first_match(view, &start, &end,
+						&start_offset, &end_offset))
+    {
+      bbs_thread_view_scroll_to_offsets(view, start_offset, end_offset);
+      view->last_match_offset = start_offset;
+      return TRUE;
+    }
+  else if (view->enable_wrap)
+    {
+      GtkTextIter iter;
+      gtk_text_buffer_get_end_iter(view->text_buffer, &iter);
+      view->last_match_offset = gtk_text_iter_get_offset(&iter);
+    }
+
+  return FALSE;
+}
+
+
+static void
+bbs_thread_view_unselect_text(BBSThreadView *view)
+{
+  GtkTextIter iter;
+  gtk_text_buffer_get_iter_at_mark(view->text_buffer, &iter,
+				   view->insert_mark);
+  gtk_text_buffer_move_mark(view->text_buffer, view->selection_bound_mark,
+			    &iter);
+}
+
+
+gboolean
+bbs_thread_view_find(BBSThreadView *view, const gchar *key,
+		     BBSThreadViewSearchDirection direction,
+		     gboolean enable_wrap, gboolean match_case,
+		     gboolean use_regexp)
+{
+  gboolean result = TRUE;
+
+  g_return_val_if_fail(IS_BBS_THREAD_VIEW(view) && key != NULL, FALSE);
+
+  /* ǽ˸νԤ */
+  view->search_direction = direction;
+  view->enable_wrap = enable_wrap;
+  view->match_case = match_case;
+
+  if (view->search_key != NULL)
+    {
+      g_free(view->search_key);
+      view->search_key = NULL;
+    }
+
+#if ENABLE_REGEXP
+  if (use_regexp)
+    {
+      int regcomp_result;
+      if (view->regexp_available)
+	regfree(&view->regexp);
+      if (match_case)
+	regcomp_result = regcomp(&view->regexp, key,
+				 REG_EXTENDED | REG_NEWLINE);
+      else
+	regcomp_result = regcomp(&view->regexp, key,
+				 REG_EXTENDED | REG_ICASE | REG_NEWLINE);
+      if (regcomp_result != 0)
+	{
+	  view->regexp_available = FALSE;
+	  regfree(&view->regexp);
+	  result = FALSE;
+	}
+      else
+	view->regexp_available = TRUE;
+    }
+  else
+    {
+      if (match_case)
+	view->search_key = g_strdup(key);
+      else
+	view->search_key = g_utf8_casefold(key, -1);
+    }
+  view->use_regexp = use_regexp;
+#else
+  if (match_case)
+    view->search_key = g_strdup(key);
+  else
+    view->search_key = g_utf8_casefold(key, -1);
+  view->use_regexp = FALSE;
+  view->regexp_available = FALSE;
+#endif
+
+  if (result)
+    {
+      if (view->search_direction == BBS_THREAD_VIEW_SEARCH_DIRECTION_FORWARD)
+	result = bbs_thread_view_do_search_forward(view, TRUE);
+      else
+	result = bbs_thread_view_do_search_backward(view, TRUE);
+    }
+
+  if (!result)
+    bbs_thread_view_unselect_text(view);
+
+  return result;
+}
+
+
+gboolean
+bbs_thread_view_find_next(BBSThreadView *view)
+{
+  gboolean result;
+  g_return_val_if_fail(IS_BBS_THREAD_VIEW(view), FALSE);
+
+  if (view->search_direction == BBS_THREAD_VIEW_SEARCH_DIRECTION_FORWARD)
+    result = bbs_thread_view_do_search_forward(view, FALSE);
+  else
+    result = bbs_thread_view_do_search_backward(view, FALSE);
+
+  if (!result)
+    bbs_thread_view_unselect_text(view);
+
+  return result;
+}
+
+
+void
+bbs_thread_view_invalidate_search_result(BBSThreadView *view)
+{
+  g_return_if_fail(IS_BBS_THREAD_VIEW(view));
+  bbs_thread_view_unselect_text(view);
+
+  if (view->search_key != NULL)
+    {
+      g_free(view->search_key);
+      view->search_key = NULL;
+    }
+
+#if ENABLE_REGEXP
+  if (view->regexp_available)
+    {
+      regfree(&view->regexp);
+      view->regexp_available = FALSE;
+    }
+#endif
+
+  view->last_match_offset = 0;
+}
