@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.h,v 1.1.1.1 2003/05/10 16:34:25 fuyu Exp $
+ * $Id: bbs_thread_view.h,v 1.3 2003/05/19 04:15:24 fuyu Exp $
  */
 
 #ifndef _BBS_THREAD_VIEW_H_
@@ -49,6 +49,21 @@ extern "C" {
 typedef struct _BBSThreadView BBSThreadView;
 typedef struct _BBSThreadViewClass BBSThreadViewClass;
 
+typedef enum {
+  BBS_THREAD_VIEW_SEARCH_DIRECTION_BACKWARD = 0,
+  BBS_THREAD_VIEW_SEARCH_DIRECTION_FORWARD = 1,
+} BBSThreadViewSearchDirection;
+
+
+typedef enum {
+  BBS_THREAD_VIEW_SEARCH_ACTION_FORWARD,
+  BBS_THREAD_VIEW_SEARCH_ACTION_BACKWARD,
+  BBS_THREAD_VIEW_SEARCH_ACTION_REGEXP_FORWARD,
+  BBS_THREAD_VIEW_SEARCH_ACTION_REGEXP_BACKWARD,
+  BBS_THREAD_VIEW_SEARCH_ACTION_TERMINATE
+} BBSThreadViewSearchAction;
+
+
 struct _BBSThreadView
 {
   GtkScrolledWindow text_container;
@@ -56,18 +71,37 @@ struct _BBSThreadView
   GtkTextView *text_view;
   GtkTextBuffer *text_buffer;
 
+  GdkCursor *ibeam_cursor;
+  GdkCursor *hand_cursor;
+
   /* Хåե˻Ȥ */
   GtkTextIter iter;
   GSList *tags;
   int alt_link_tag;
 
+
   /* ʥطǻȤ */
   gint last_offset;
   OchushaBBSThread *thread;
   GHashTable *link_table;	/* offset=>linkʥơ֥ */
 
-  GdkCursor *ibeam_cursor;
-  GdkCursor *hand_cursor;
+
+  /* طǻȤ */
+  gint last_match_offset;
+  guint idle_search_func;
+  GtkTextMark *insert_mark;
+  GtkTextMark *selection_bound_mark;
+
+
+  gchar *search_key;
+#if ENABLE_REGEXP
+  regex_t regexp;
+#endif
+  guint regexp_available	: 1;
+  guint search_direction	: 1;
+  guint enable_wrap		: 1;
+  guint match_case		: 1;
+  guint use_regexp		: 1;
 };
 
 
@@ -139,6 +173,15 @@ void bbs_thread_view_place_cursor_onscreen(BBSThreadView *view);
 
 void bbs_thread_view_hide_scrollbar(BBSThreadView *view);
 
+gboolean bbs_thread_view_find(BBSThreadView *view, const gchar *key,
+			      BBSThreadViewSearchDirection direction,
+			      gboolean enable_wrap, gboolean match_case,
+			      gboolean use_regexp);
+gboolean bbs_thread_view_find_next(BBSThreadView *view);
+void bbs_thread_view_invalidate_search_result(BBSThreadView *view);
+
+
+
 #ifdef __cplusplus
 } // extern "C"
 #endif
