@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: board_properties.c,v 1.1.1.1 2003/05/10 16:34:25 fuyu Exp $
+ * $Id: board_properties.c,v 1.2 2003/05/14 20:58:12 fuyu Exp $
  */
 
 #include "ochusha_ui.h"
@@ -109,7 +109,7 @@ open_board_properties(OchushaApplication *application,
   GtkWidget *frame;
   GtkWidget *tmp_widget;
   DialogStates *states;
-  BulletinBoardGUIInfo *info = (BulletinBoardGUIInfo *)board->user_data;
+  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board);
 
   BOARD_PROPERTIES_LOCK;
 
