@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.c,v 1.2 2003/05/11 20:55:12 fuyu Exp $
+ * $Id: boardlist_ui.c,v 1.10 2003/05/19 04:14:55 fuyu Exp $
  */
 
 #include "config.h"
@@ -39,6 +39,7 @@
 #include "boardlist_view.h"
 #include "bulletin_board_ui.h"
 #include "paned_notebook.h"
+#include "text_search_window.h"
 #include "ugly_gtk2utils.h"
 
 #include "worker.h"
@@ -57,6 +58,13 @@ static void select_category_cb(BoardlistView *view,
 			       OchushaBoardCategory *category,
 			       gpointer user_data);
 #endif
+static void category_expanded_cb(BoardlistView *view,
+				 OchushaBoardCategory *category,
+				 OchushaApplication *application);
+static void category_collapsed_cb(BoardlistView *view,
+				  OchushaBoardCategory *category,
+				  OchushaApplication *application);
+
 static void boardlist_read_category_element_cb(OchushaBBSTable *table,
 					       OchushaBoardCategory *category,
 					       GHashTable *cat_attributes,
@@ -132,6 +140,33 @@ static pthread_mutex_t board_list_lock;
     }
 
 
+typedef struct _BoardCategoryGUIInfo
+{
+  gboolean expanded;
+} BoardCategoryGUIInfo;
+
+
+static GQuark category_gui_info_id;
+
+
+void
+prepare_boardlist_ui_initialization(OchushaApplication *application)
+{
+  if (pthread_mutex_init(&board_list_lock, NULL) != 0)
+    {
+      fprintf(stderr, "Couldn't init a mutex.\n");
+      abort();
+    }
+
+  category_gui_info_id
+    = g_quark_from_static_string("BoardlistUI::CategoryInfo");
+
+#if HAVE_ONIGURUMA
+  reg_set_encoding(REG_ENCODING_UTF8);
+#endif
+}
+
+
 void
 initialize_boardlist(OchushaApplication *application)
 {
@@ -140,13 +175,11 @@ initialize_boardlist(OchushaApplication *application)
 
   board_list = NULL;
 
-  if (pthread_mutex_init(&board_list_lock, NULL) != 0)
-    {
-      fprintf(stderr, "Couldn't init a mutex.\n");
-      abort();
-    }
-
   application->boardlist_view = boardlist_view_new();
+  g_signal_connect(G_OBJECT(application->boardlist_view), "category_expanded",
+		   G_CALLBACK(category_expanded_cb), application);
+  g_signal_connect(G_OBJECT(application->boardlist_view), "category_collapsed",
+		   G_CALLBACK(category_collapsed_cb), application);
   gtk_widget_show(application->boardlist_view);
 
   application->contents_window
@@ -221,6 +254,8 @@ initialize_boardlist(OchushaApplication *application)
 		   "page_switched", G_CALLBACK(page_switched_cb),
 		   application);
 
+  application->threadlist_search_window = NULL;
+
   application->table = ochusha_bbs_table_new();
 
   /* γĥǥ¸ */
@@ -252,6 +287,22 @@ initialize_boardlist(OchushaApplication *application)
 }
 
 
+static BoardCategoryGUIInfo *
+ensure_board_category_info(OchushaBoardCategory *category)
+{
+  BoardCategoryGUIInfo *info = g_object_get_qdata(G_OBJECT(category),
+						  category_gui_info_id);
+  if (info == NULL)
+    {
+      info = g_new0(BoardCategoryGUIInfo, 1);
+      g_object_set_qdata_full(G_OBJECT(category), category_gui_info_id,
+			      info, (GDestroyNotify)g_free);
+    }
+
+  return info;
+}
+
+
 void
 refresh_boardlist(OchushaApplication *application)
 {
@@ -400,26 +451,235 @@ redraw_current_threadlist_all(OchushaApplication *application)
 }
 
 
-void
-start_current_threadlist_search(OchushaApplication *application)
+static ThreadlistSearchQueryData search_query_data =
+{
+  NULL,		/* key */
+  TEXT_SEARCH_DIRECTION_DOWN,
+  NULL,		/* last_view */
+#if ENABLE_REGEXP
+  {},		/* regexp */
+  FALSE,	/* regexp_available */
+#endif
+  FALSE,	/* enable_wrap */
+  FALSE,	/* match_case */
+  FALSE,	/* use_regexp */
+};
+
+
+static void
+text_search_window_response_cb(TextSearchWindow *window, gint response_id,
+			       OchushaApplication *application)
+{
+  switch (response_id)
+    {
+    case GTK_RESPONSE_DELETE_EVENT:
+      application->threadlist_search_window = NULL;
+      /* fall through */
+
+    case GTK_RESPONSE_CANCEL:
+      /* XXX: ɥξȥϿ٤ */
+      if (search_query_data.last_view != NULL)
+	threadlist_view_set_being_searched(search_query_data.last_view, FALSE);
+      search_query_data.last_view = NULL;
+      gtk_widget_hide(GTK_WIDGET(window));
+      return;
+    }
+}
+
+
+static gboolean
+do_search_on_current_threadlist(OchushaApplication *application,
+				gboolean include_current_pos)
 {
   GtkWidget *widget;
+  ThreadlistView *view;
 
-  g_return_if_fail(application != NULL
-		   && application->contents_window != NULL);
+  g_return_val_if_fail(application != NULL
+		       && application->contents_window != NULL, FALSE);
 
   widget = paned_notebook_get_current_item_view(
 				PANED_NOTEBOOK(application->contents_window));
   if (widget == NULL)
-    return;
-  g_return_if_fail(IS_PANED_NOTEBOOK(widget));
+    return FALSE;
+  g_return_val_if_fail(IS_PANED_NOTEBOOK(widget), FALSE);
 
   widget = paned_notebook_get_selector(PANED_NOTEBOOK(widget));
   if (widget == NULL)
-    return;
-  g_return_if_fail(IS_THREADLIST_VIEW(widget));
+    return FALSE;
+  g_return_val_if_fail(IS_THREADLIST_VIEW(widget), FALSE);
+  view = THREADLIST_VIEW(widget);
+  search_query_data.last_view = view;
+  threadlist_view_set_being_searched(view, TRUE);
 
-  threadlist_view_start_interactive_search(THREADLIST_VIEW(widget));
+  return threadlist_view_find_next(view,
+				   search_query_data.direction,
+				   search_query_data.enable_wrap,
+				   include_current_pos,
+				   &search_query_data);
+}
+
+
+static gboolean
+text_search_window_query_changed_cb(TextSearchWindow *window, const gchar *key,
+				    TextSearchDirection direction,
+				    gboolean enable_wrap, gboolean match_case,
+				    gboolean use_regexp,
+				    OchushaApplication *application)
+{
+  gboolean result;
+  gchar *normalized_key;
+
+  if (search_query_data.key != NULL)
+    g_free(search_query_data.key);
+
+  normalized_key = g_utf8_normalize(key, -1, G_NORMALIZE_ALL);
+  if (!match_case)
+    {
+      gchar *case_normalized_key = g_utf8_casefold(normalized_key, -1);
+      g_free(normalized_key);
+      normalized_key = case_normalized_key;
+    }
+  search_query_data.key = normalized_key;
+  search_query_data.direction = direction;
+  search_query_data.enable_wrap = enable_wrap;
+  search_query_data.match_case = match_case;
+#if ENABLE_REGEXP
+  search_query_data.use_regexp = use_regexp;
+  if (use_regexp)
+    {
+      if (search_query_data.regexp_available)
+	regfree(&search_query_data.regexp);
+
+      if (regcomp(&search_query_data.regexp,
+		  normalized_key, REG_EXTENDED | REG_NEWLINE | REG_NOSUB) == 0)
+	{
+	  search_query_data.regexp_available = TRUE;
+	  result = do_search_on_current_threadlist(application, TRUE);
+	}
+      else
+	{
+	  regfree(&search_query_data.regexp);
+	  search_query_data.regexp_available = FALSE;
+	  result = FALSE;
+	}
+    }
+  else
+    result = do_search_on_current_threadlist(application, TRUE);
+#else
+  search_query_data.use_regexp = FALSE;
+  result = do_search_on_current_threadlist(application, TRUE);
+#endif
+
+  return result;
+}
+
+
+static gboolean
+text_search_window_find_next_cb(TextSearchWindow *window, const gchar *key,
+				TextSearchDirection direction,
+				gboolean enable_wrap, gboolean match_case,
+				gboolean use_regexp,
+				OchushaApplication *application)
+{
+  return do_search_on_current_threadlist(application, FALSE);
+}
+
+
+static void
+setup_threadlist_search_window(OchushaApplication *application)
+{
+  if (search_query_data.key != NULL)
+    {
+      g_free(search_query_data.key);
+      search_query_data.key = NULL;
+    }
+
+  application->threadlist_search_window = text_search_window_new();
+  text_search_window_set_enable_incremental_search(
+		TEXT_SEARCH_WINDOW(application->threadlist_search_window),
+		TRUE);
+  g_signal_connect(application->threadlist_search_window, "response",
+		   G_CALLBACK(text_search_window_response_cb),
+		   application);
+  g_signal_connect(application->threadlist_search_window, "query_changed",
+		   G_CALLBACK(text_search_window_query_changed_cb),
+		   application);
+  g_signal_connect(application->threadlist_search_window, "find_next",
+		   G_CALLBACK(text_search_window_find_next_cb),
+		   application);
+  gtk_window_set_title(GTK_WINDOW(application->threadlist_search_window),
+		       _("Find in the current threadlist"));
+  gtk_window_set_transient_for(
+			GTK_WINDOW(application->threadlist_search_window),
+			application->top_level);
+}
+
+
+void
+advance_threadlist_search(OchushaApplication *application,
+			  ThreadlistView *view,
+			  ThreadlistSearchAction search_action)
+{
+  switch (search_action)
+    {
+    case THREADLIST_SEARCH_ACTION_FORWARD:
+    case THREADLIST_SEARCH_ACTION_REGEXP_FORWARD:
+      search_query_data.direction = THREADLIST_SEARCH_DIRECTION_FORWARD;
+      break;
+
+    case THREADLIST_SEARCH_ACTION_BACKWARD:
+    case THREADLIST_SEARCH_ACTION_REGEXP_BACKWARD:
+      search_query_data.direction = THREADLIST_SEARCH_DIRECTION_BACKWARD;
+      break;
+
+    default:
+      /* SHOULD NOT reached here */
+      /* THREADLIST_SEARCH_ACTION_TERMINATEcaller¦ǽ */
+    }
+
+  if (application->threadlist_search_window == NULL
+      || !GTK_WIDGET_VISIBLE(view)
+      || search_query_data.last_view != view)
+    {
+      if (search_query_data.last_view != NULL)
+	threadlist_view_set_being_searched(search_query_data.last_view,
+					   FALSE);
+      start_current_threadlist_search(application);
+      search_query_data.last_view = view;
+    }
+  else
+    do_search_on_current_threadlist(application,
+				    search_query_data.last_view != view);
+}
+
+
+void
+terminate_threadlist_search(OchushaApplication *application,
+			    ThreadlistView *view)
+{
+  TextSearchWindow *window;
+  window = TEXT_SEARCH_WINDOW(application->threadlist_search_window);
+  if (window != NULL)
+    g_object_ref(G_OBJECT(window));
+
+  if (window != NULL)
+    {
+      text_search_window_response(window, GTK_RESPONSE_CANCEL);
+      g_object_unref(G_OBJECT(window));
+    }
+}
+
+
+void
+start_current_threadlist_search(OchushaApplication *application)
+{
+  if (application->threadlist_search_window == NULL)
+    setup_threadlist_search_window(application);
+
+  text_search_window_set_key_selected(
+		TEXT_SEARCH_WINDOW(application->threadlist_search_window));
+  gtk_widget_show(application->threadlist_search_window);
+  gtk_widget_grab_focus(application->threadlist_search_window);
 }
 
 
@@ -440,7 +700,7 @@ fix_filter_button_state(OchushaApplication *application)
     return;
 
   gtk_toggle_button_set_active(application->filter_button,
-		((BulletinBoardGUIInfo *)board->user_data)->enable_filter);
+			ensure_bulletin_board_info(board)->enable_filter);
 }
 
 
@@ -485,18 +745,10 @@ filter_toggled_cb(GtkToggleButton *toggle_button,
     = (OchushaBulletinBoard *)paned_notebook_get_current_item(board_pane);
   BulletinBoardGUIInfo *info;
 
-#if DEBUG_GUI_MOST
-  fprintf(stderr, "filter_toggled_cb.\n");
-#endif
-
   if (board == NULL)
     return;
 
-  info = (BulletinBoardGUIInfo *)board->user_data;
-
-  if (info == NULL)
-    return;
-
+  info = ensure_bulletin_board_info(board);
   if (info->enable_filter == gtk_toggle_button_get_active(toggle_button))
     return;
 
@@ -527,9 +779,6 @@ item_view_required_cb(PanedNotebook *paned_notebook, OchushaBulletinBoard *board
 		      OchushaApplication *application)
 {
   OchushaBulletinBoard *board2;
-#if DEBUG_GUI_MOST
-  fprintf(stderr, "boardlist_ui.c: item_view_required_cb called.\n");
-#endif
   *tab_label = get_tab_label(board->name);
   *item_view = open_bulletin_board(application, board, ICON_LABEL(*tab_label));
 
@@ -626,9 +875,6 @@ static void
 item_view_closed_cb(PanedNotebook *paned_notebook, OchushaBulletinBoard *board,
 		    GtkWidget *item_view, OchushaApplication *application)
 {
-#if DEBUG_GUI_MOST
-  fprintf(stderr, "boardlist_ui.c: item_view_closed_cb called.\n");
-#endif
   if (paned_notebook_get_current_item_view(paned_notebook) == NULL)
     {
       /* ɽˤǤƤڡĤ줿 */
@@ -651,16 +897,24 @@ page_switched_cb(PanedNotebook *paned_notebook,
 		 GtkWidget *previous_page, GtkWidget *new_page,
 		 OchushaApplication *application)
 {
-#if DEBUG_GUI_MOST
-  fprintf(stderr, "boardlist_ui.c: page_switched_cb()\n");
-#endif
   g_return_if_fail((previous_page != NULL && IS_PANED_NOTEBOOK(previous_page))
 		   || (new_page != NULL && IS_PANED_NOTEBOOK(new_page)));
 
   if (previous_page != NULL)
     {
+      gpointer threadlist_view;
       application->threadlist_height
 	= gtk_paned_get_position(GTK_PANED(previous_page));
+      threadlist_view
+	= paned_notebook_get_selector(PANED_NOTEBOOK(previous_page));
+
+      if (IS_THREADLIST_VIEW(threadlist_view)
+	  && search_query_data.last_view == threadlist_view)
+	{
+	  threadlist_view_set_being_searched(THREADLIST_VIEW(threadlist_view),
+					     FALSE);
+	  search_query_data.last_view = NULL;
+	}
     }
 
   if (new_page != NULL)
@@ -670,15 +924,12 @@ page_switched_cb(PanedNotebook *paned_notebook,
       BulletinBoardGUIInfo *info;
 
       g_return_if_fail(board != NULL);
-      info = (BulletinBoardGUIInfo *)board->user_data;
+      info = ensure_bulletin_board_info(board);
       gtk_paned_set_position(GTK_PANED(new_page),
 			     application->threadlist_height);
       gtk_toggle_button_set_active(application->filter_button,
 				   info->enable_filter);
     }
-#if DEBUG_GUI_MOST
-  fprintf(stderr, "boardlist_ui.c: page_switched_cb() done.\n");
-#endif
 }
 
 
@@ -688,18 +939,9 @@ boardlist_read_category_element_cb(OchushaBBSTable *table,
 				   GHashTable *category_attributes,
 				   OchushaApplication *application)
 {
-  BoardCategoryInfo *info = category->user_data;
-#if DEBUG_GUI
-  fprintf(stderr, "boardlist_read_category_element_cb.\n");
-#endif
-
-  if (info == NULL)
-    {
-      info = g_new0(BoardCategoryInfo, 1);
-      category->user_data = info;
-    }
-
-  info->expanded = ochusha_utils_get_attribute_boolean(category_attributes, "expanded");
+  BoardCategoryGUIInfo *info = ensure_board_category_info(category);
+  info->expanded = ochusha_utils_get_attribute_boolean(category_attributes,
+						       "expanded");
   return;
 }
 
@@ -738,10 +980,7 @@ boardlist_write_category_element_cb(OchushaBBSTable *table,
 				    FILE *boardlist_xml,
 				    OchushaApplication *application)
 {
-  BoardCategoryInfo *info = (BoardCategoryInfo *)category->user_data;
-#if DEBUG_GUI
-  fprintf(stderr, "boardlist_write_category_element_cb.\n");
-#endif
+  BoardCategoryGUIInfo *info = ensure_board_category_info(category);
 
   OUTPUT_CATEGORY_ATTRIBUTE_BOOLEAN(boardlist_xml, info, expanded);
 }
@@ -773,7 +1012,7 @@ boardlist_write_board_element_cb(OchushaBBSTable *table,
 				 FILE *boardlist_xml,
 				 OchushaApplication *application)
 {
-  BulletinBoardGUIInfo *info = (BulletinBoardGUIInfo *)board->user_data;
+  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board);
   OUTPUT_BOARD_ATTRIBUTE_BOOLEAN(boardlist_xml, info, enable_filter);
 
   if (info->last_modified != NULL)
@@ -795,23 +1034,27 @@ write_threadlist(gpointer data, gpointer user_data)
   OchushaBulletinBoard *board = (OchushaBulletinBoard *)data;
   OchushaApplication *application = (OchushaApplication *)user_data;
 
+  /* XXX: 顼 */
   if (!ochusha_bulletin_board_write_threadlist_xml(board, &application->conf))
     fprintf(stderr, "Couldn't write threadlist2.xml.\n");
 }
 
 
-BoardCategoryInfo *
-ensure_board_category_info(OchushaBoardCategory *category)
+static void
+category_expanded_cb(BoardlistView *view, OchushaBoardCategory *category,
+		     OchushaApplication *application)
 {
-  BoardCategoryInfo *info = category->user_data;
+  BoardCategoryGUIInfo *info = ensure_board_category_info(category);
+  info->expanded = TRUE;
+}
 
-  if (info == NULL)
-    {
-      info = g_new0(BoardCategoryInfo, 1);
-      category->user_data = info;
-    }
 
-  return info;
+static void
+category_collapsed_cb(BoardlistView *view, OchushaBoardCategory *category,
+		      OchushaApplication *application)
+{
+  BoardCategoryGUIInfo *info = ensure_board_category_info(category);
+  info->expanded = FALSE;
 }
 
 
@@ -840,11 +1083,6 @@ build_boardlist(WorkerThread *employee, gpointer args)
   OchushaApplication *application = (OchushaApplication *)args;
   OchushaBBSTable *table = application->table;
 
-#if DEBUG_THREAD_MOST
-  fprintf(stderr, "build_boardlist thread is invoked.\n");
-  fflush(stderr);
-#endif
-
   if ((status = pthread_mutex_trylock(&board_list_lock)) != 0)
     {
       if (status == EBUSY)
@@ -858,6 +1096,7 @@ build_boardlist(WorkerThread *employee, gpointer args)
 
   if (buffer == NULL)
     {
+      /* XXX: 顼 */
       fprintf(stderr, "Couldn't get %s\n", application->conf.bbsmenu_url);
       goto cleanup_before_return;
       return;
@@ -888,15 +1127,23 @@ build_boardlist(WorkerThread *employee, gpointer args)
 }
 
 
+static void
+restore_expanded_state(OchushaBoardCategory *category, BoardlistView *view)
+{
+  BoardCategoryGUIInfo *info = ensure_board_category_info(category);
+  if (info->expanded)
+    boardlist_view_expand_category(view, category);
+}
+
+
 static void
 setup_boardlist(OchushaApplication *application)
 {
-#if DEBUG_GUI_MOST
-  fprintf(stderr, "setup_boardlist\n");
-#endif
   boardlist_view_open(BOARDLIST_VIEW(application->boardlist_view),
-		      application->table->category_list, NULL,
-		      (BulletinBoardInfoNewFunc *)ensure_bulletin_board_info);
+		      application->table->category_list);
+  g_slist_foreach(application->table->category_list,
+		  (GFunc)restore_expanded_state, application->boardlist_view);
+
   gtk_widget_show_all(application->boardlist_view);
   gtk_widget_show_all(application->contents_window);
 }
@@ -915,9 +1162,6 @@ static void
 open_board_properties_button_cb(GtkWidget *widget,
 				OchushaApplication *application)
 {
-#if DEBUG_GUI_MOST
-  fprintf(stderr, "open_board_properties_button_cb: widget=0x%x\n", (int)widget);
-#endif
   open_current_board_properties(application);
 }
 
@@ -925,9 +1169,6 @@ open_board_properties_button_cb(GtkWidget *widget,
 static void
 refresh_board_button_cb(GtkWidget *widget, OchushaApplication *application)
 {
-#if DEBUG_GUI_MOST
-  fprintf(stderr, "refresh_board_button_cb: widget=0x%x\n", (int)widget);
-#endif
   refresh_current_threadlist(application);
 }
 
