@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.h,v 1.2 2003/05/11 20:55:12 fuyu Exp $
+ * $Id: boardlist_ui.h,v 1.7 2003/05/17 08:14:47 fuyu Exp $
  */
 
 #ifndef _BOARDLIST_UI_H_
@@ -43,13 +43,12 @@ extern "C" {
 /*
  * İGUIϢ
  */
+void prepare_boardlist_ui_initialization(OchushaApplication *application);
 void initialize_boardlist(OchushaApplication *application);
 void refresh_boardlist(OchushaApplication *application);
 void refresh_boardlist2(OchushaApplication *application);
 void finalize_boardlist(OchushaApplication *application);
 
-BoardCategoryInfo *ensure_board_category_info(OchushaBoardCategory *category);
-
 void update_threadlist_entry(OchushaApplication *application,
 			     OchushaBBSThread *thread);
 
@@ -60,6 +59,11 @@ void refresh_current_threadlist_all(OchushaApplication *application);
 void redraw_current_threadlist_all(OchushaApplication *application);
 void fix_filter_button_state(OchushaApplication *application);
 
+void advance_threadlist_search(OchushaApplication *application,
+			       ThreadlistView *view,
+			       ThreadlistSearchAction search_action);
+void terminate_threadlist_search(OchushaApplication *application,
+				 ThreadlistView *view);
 void start_current_threadlist_search(OchushaApplication *application);
 
 #ifdef __cplusplus
