@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_view.c,v 1.1.1.1 2003/05/10 16:34:25 fuyu Exp $
+ * $Id: boardlist_view.c,v 1.4 2003/05/15 19:33:49 fuyu Exp $
  */
 
 #include "config.h"
@@ -64,8 +64,6 @@ static gboolean tree_view_button_release_event_cb(GtkWidget *widget,
 						  BoardlistView *view);
 static void append_board(gpointer data, gpointer user_data);
 static void append_category(gpointer data, gpointer user_data);
-static void restore_expanded_state(OchushaBoardCategory *category,
-				   BoardlistView *view);
 
 
 GType
@@ -96,6 +94,12 @@ boardlist_view_get_type(void)
 }
 
 
+typedef struct _TreeNodeInfo
+{
+  GtkTreeIter iter;
+} TreeNodeInfo;
+
+
 enum {
   OPEN_VIEW_SIGNAL,
 
@@ -108,6 +112,22 @@ enum {
 
 static GtkScrolledWindowClass *parent_class = NULL;
 static gint boardlist_view_signals[LAST_SIGNAL] = { 0, 0, 0, 0 };
+static GQuark node_info_id;
+
+
+static TreeNodeInfo *
+ensure_node_info(GObject *object)
+{
+  TreeNodeInfo *info = g_object_get_qdata(object, node_info_id);
+  if (info == NULL)
+    {
+      info = g_new0(TreeNodeInfo, 1);
+      g_object_set_qdata_full(object, node_info_id,
+			      info, (GDestroyNotify)g_free);
+    }
+
+  return info;
+}
 
 
 static void
@@ -149,23 +169,25 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(BoardlistViewClass, category_expanded),
 		 NULL, NULL,
-		 ochusha_marshal_VOID__POINTER,
+		 ochusha_marshal_VOID__OBJECT,
 		 G_TYPE_NONE, 1,
-		 G_TYPE_POINTER);
+		 OCHUSHA_TYPE_BOARD_CATEGORY);
   boardlist_view_signals[CATEGORY_COLLAPSED_SIGNAL] =
     g_signal_new("category_collapsed",
 		 G_TYPE_FROM_CLASS(klass),
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(BoardlistViewClass, category_collapsed),
 		 NULL, NULL,
-		 ochusha_marshal_VOID__POINTER,
+		 ochusha_marshal_VOID__OBJECT,
 		 G_TYPE_NONE, 1,
-		 G_TYPE_POINTER);
+		 OCHUSHA_TYPE_BOARD_CATEGORY);
 
   klass->open_view = NULL;
   klass->select_category = NULL;
   klass->category_expanded = NULL;
   klass->category_collapsed = NULL;
+
+  node_info_id = g_quark_from_static_string("BoardlistView::TreeNodeInfo");
 }
 
 
@@ -267,7 +289,7 @@ boardlist_view_expand_category(BoardlistView *view,
 			       OchushaBoardCategory *category)
 {
   gboolean expanded;
-  BoardCategoryInfo *info = (BoardCategoryInfo *)category->user_data;
+  TreeNodeInfo *info = ensure_node_info(G_OBJECT(category));
   GtkTreePath *path = gtk_tree_model_get_path(GTK_TREE_MODEL(view->model),
 					      &info->iter);
   expanded = gtk_tree_view_expand_row(view->view, path, FALSE);
@@ -287,7 +309,7 @@ boardlist_view_collapse_category(BoardlistView *view,
 				 OchushaBoardCategory *category)
 {
   gboolean collapsed;
-  BoardCategoryInfo *info = (BoardCategoryInfo *)category->user_data;
+  TreeNodeInfo *info = ensure_node_info(G_OBJECT(category));
   GtkTreePath *path = gtk_tree_model_get_path(GTK_TREE_MODEL(view->model),
 					      &info->iter);
   collapsed = gtk_tree_view_collapse_row(view->view, path);
@@ -409,8 +431,6 @@ tree_view_row_expanded_cb(GtkTreeView *tree_view,
 
   if (category != NULL)
     {
-      BoardCategoryInfo *info = category->user_data;
-      info->expanded = TRUE;
       g_signal_emit(G_OBJECT(view),
 		    boardlist_view_signals[CATEGORY_EXPANDED_SIGNAL],
 		    0,
@@ -447,8 +467,6 @@ tree_view_row_collapsed_cb(GtkTreeView *tree_view,
 
   if (category != NULL)
     {
-      BoardCategoryInfo *info = category->user_data;
-      info->expanded = FALSE;
       g_signal_emit(G_OBJECT(view),
 		    boardlist_view_signals[CATEGORY_COLLAPSED_SIGNAL],
 		    0,
@@ -461,8 +479,6 @@ typedef struct _AppendBoardArgs
 {
   GtkTreeIter *category_iter;
   GtkTreeStore *store;
-  BoardCategoryInfoNewFunc *category_info_factory;
-  BulletinBoardInfoNewFunc *board_info_factory;
 } AppendBoardArgs;
 
 
@@ -471,16 +487,7 @@ append_board(gpointer data, gpointer user_data)
 {
   OchushaBulletinBoard *board = (OchushaBulletinBoard *)data;
   AppendBoardArgs *args = (AppendBoardArgs *)user_data;
-  BulletinBoardInfo *info = (BulletinBoardInfo *)board->user_data;
-
-  if (info == NULL)
-    {
-      if (args->board_info_factory != NULL)
-	info = (*args->board_info_factory)(board);
-      else
-	info = (BulletinBoardInfo *)calloc(1, sizeof(BulletinBoardInfo));
-      board->user_data = info;
-    }
+  TreeNodeInfo *info = ensure_node_info(G_OBJECT(board));
 
   gtk_tree_store_append(args->store, &info->iter, args->category_iter);
   gtk_tree_store_set(args->store, &info->iter,
@@ -495,18 +502,9 @@ static void
 append_category(gpointer data, gpointer user_data)
 {
   OchushaBoardCategory *category = (OchushaBoardCategory *)data;
-  BoardCategoryInfo *info = (BoardCategoryInfo *)category->user_data;
+  TreeNodeInfo *info = ensure_node_info(G_OBJECT(category));
   AppendBoardArgs *args = (AppendBoardArgs *)user_data;
 
-  if (info == NULL)
-    {
-      if (args->category_info_factory != NULL)
-	info = (*args->category_info_factory)(category);
-      else
-	info = (BoardCategoryInfo *)calloc(1, sizeof(BoardCategoryInfo));
-      category->user_data = info;
-    }
-
   args->category_iter = &info->iter;
 
   gtk_tree_store_append(args->store, args->category_iter, NULL);
@@ -520,27 +518,15 @@ append_category(gpointer data, gpointer user_data)
 }
 
 
-static void
-restore_expanded_state(OchushaBoardCategory *category, BoardlistView *view)
-{
-  BoardCategoryInfo *info = (BoardCategoryInfo *)category->user_data;
-  if (info->expanded)
-    boardlist_view_expand_category(view, category);
-}
-
-
 void
-boardlist_view_open(BoardlistView *view, GSList *category_list,
-		    BoardCategoryInfoNewFunc *category_info_factory,
-		    BulletinBoardInfoNewFunc *board_info_factory)
+boardlist_view_open(BoardlistView *view, GSList *category_list)
 {
   GtkTreeStore *store = gtk_tree_store_new(BOARDLIST_N_COLUMNS,
 		/* tree view label */	   G_TYPE_STRING,
 		/* row type */		   G_TYPE_INT,
 		/* data */		   G_TYPE_POINTER);
   		/* (OchushaBoardCategory *)  (OchushaBulletinBoard *) */
-  AppendBoardArgs args = { NULL, store,
-			   category_info_factory, board_info_factory };
+  AppendBoardArgs args = { NULL, store };
 
   g_slist_foreach(category_list, append_category, &args);
 
@@ -549,8 +535,6 @@ boardlist_view_open(BoardlistView *view, GSList *category_list,
   if (view->model != NULL)
     g_object_unref(G_OBJECT(view->model));
   view->model = GTK_TREE_MODEL(store);
-
-  g_slist_foreach(category_list, (GFunc)restore_expanded_state, view);
 }
 
 
