@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.4 2003/05/11 18:20:36 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.14 2003/05/19 04:15:24 fuyu Exp $
  */
 
 #include "config.h"
@@ -39,6 +39,7 @@
 #include "boardlist_ui.h"
 #include "bulletin_board_ui.h"
 #include "icon_label.h"
+#include "text_search_window.h"
 #include "threadlist_filter.h"
 #include "threadlist_view.h"
 #include "paned_notebook.h"
@@ -100,6 +101,8 @@ static void write_response_button_cb(GtkWidget *widget,
 				     OchushaApplication *application);
 static void refresh_thread_button_cb(GtkWidget *widget,
 				     OchushaApplication *application);
+static void start_thread_search_button_cb(GtkWidget *widget,
+					  OchushaApplication *application);
 static void jump_to_new_comer_button_cb(GtkWidget *widget,
 					OchushaApplication *application);
 static void go_to_bottom_button_cb(GtkWidget *widget,
@@ -150,10 +153,13 @@ static gint title_popup_y_pos = 0;
 static guint title_popup_delay_id = 0;
 
 
+static GQuark board_info_id;
+
+
 void
-initialize_board_ui(OchushaApplication *application)
+prepare_board_ui_initialization(OchushaApplication *application)
 {
-  if (pthread_mutex_init(&thread_list_lock, NULL) != 0)
+ if (pthread_mutex_init(&thread_list_lock, NULL) != 0)
     {
       fprintf(stderr, "Couldn't init a mutex.\n");
       abort();
@@ -165,6 +171,13 @@ initialize_board_ui(OchushaApplication *application)
       abort();
     }
 
+  board_info_id = g_quark_from_static_string("BulletinBoardUI::BoardInfo");
+}
+
+
+void
+initialize_board_ui(OchushaApplication *application)
+{
   setup_default_threadlist_filter(application);
   initialize_board_properties(application);
 
@@ -184,6 +197,8 @@ initialize_board_ui(OchushaApplication *application)
   gtk_widget_show(title_popup_label);
 
   gtk_container_add(GTK_CONTAINER(title_popup_window), title_popup_label);
+
+  application->thread_search_window = NULL;
 }
 
 
@@ -195,10 +210,9 @@ threadlist_read_thread_element_cb(OchushaBulletinBoard *board,
 {
   BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
 
-  info->info.view_flags = ochusha_utils_get_attribute_int(hash_table,
-							  "view_flags");
-  info->info.view_rank = ochusha_utils_get_attribute_int(hash_table,
-							 "view_rank");
+  info->view_flags = ochusha_utils_get_attribute_int(hash_table, "view_flags");
+  info->view_rank = ochusha_utils_get_attribute_int(hash_table, "view_rank");
+
   info->last_modified = ochusha_utils_get_attribute_string(hash_table,
 							   "last_modified");
   info->view_ignored = ochusha_utils_get_attribute_int(hash_table,
@@ -226,14 +240,14 @@ threadlist_write_thread_element_cb(OchushaBulletinBoard *board,
 				   OchushaBBSThread *thread,
 				   FILE *threadlist_xml, gpointer user_data)
 {
-  BBSThreadGUIInfo *info = (BBSThreadGUIInfo *)thread->user_data;
-  ThreadlistInfo *threadlist_info;
-  if (info != NULL)
-    {
-      threadlist_info = &info->info;
-      OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, threadlist_info, view_flags);
-      OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, threadlist_info, view_rank);
+  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
 
+  if (info->view_rank != 0)
+    {
+      info->view_flags &= ~BBS_THREAD_NEW;
+      OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, info,
+				  view_flags);
+      OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, info, view_rank);
       if (info->last_modified != NULL)
 	OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, info, last_modified);
 
@@ -248,11 +262,12 @@ ensure_bulletin_board_info(OchushaBulletinBoard *board)
   BulletinBoardGUIInfo *info;
   THREAD_LIST_LOCK
     {
-      info = (BulletinBoardGUIInfo *)board->user_data;
+      info = g_object_get_qdata(G_OBJECT(board), board_info_id);
       if (info == NULL)
 	{
 	  info = g_new0(BulletinBoardGUIInfo, 1);
-	  board->user_data = info;
+	  g_object_set_qdata_full(G_OBJECT(board), board_info_id,
+				  info, g_free);
 	  g_signal_connect(G_OBJECT(board), "threadlist_read_thread_element",
 			   G_CALLBACK(threadlist_read_thread_element_cb),
 			   NULL);
@@ -277,6 +292,74 @@ typedef struct _ThreadlistJobArgs
 } ThreadlistJobArgs;
 
 
+static gboolean
+interactive_search_cb(ThreadlistView *view,
+		      ThreadlistSearchAction search_action,
+		      OchushaApplication *application)
+{
+  if (search_action != THREADLIST_SEARCH_ACTION_TERMINATE)
+    advance_threadlist_search(application, view, search_action);
+  else
+    terminate_threadlist_search(application, view);
+
+  return TRUE;
+}
+
+
+static gboolean
+title_match_func(const gchar *title, ThreadlistSearchQueryData *search_data)
+{
+  gboolean result;
+  gchar *normalized_title;
+
+#if DEBUG_SEARCH
+  char *native_string;
+  if (title != NULL)
+    {
+      native_string = convert_string(utf8_to_native, title, -1);
+      fprintf(stderr, "title_match_func: title=\"%s\"\n", native_string);
+      g_free(native_string);
+    }
+  if (search_data->key != NULL)
+    {
+      native_string = convert_string(utf8_to_native, search_data->key, -1);
+      fprintf(stderr, "title_match_func: key=\"%s\"\n", native_string);
+      g_free(native_string);
+    }
+#endif
+  if (search_data->key == NULL)
+    return FALSE;
+
+  /* match_case˹碌normalizeƤ롣*/
+  normalized_title = g_utf8_normalize(title, -1, G_NORMALIZE_ALL);
+  if (!search_data->match_case)
+    {
+      gchar *case_normalized_title = g_utf8_casefold(normalized_title, -1);
+      g_free(normalized_title);
+      normalized_title = case_normalized_title;
+    }
+
+#if ENABLE_REGEXP
+  if (search_data->use_regexp)
+    {
+      if (search_data->regexp_available)
+	result = (regexec(&search_data->regexp, normalized_title,
+			  0, NULL, 0) == 0);
+      else
+	result = TRUE;	/* ̵̤ʸ򤵤ʤ */
+    }
+  else
+    result = (strstr(normalized_title, search_data->key) != NULL);
+#else
+  result = (strstr(normalized_title, search_data->key) != NULL);
+#endif
+
+  g_free(normalized_title);
+
+  return result;
+}
+
+
 GtkWidget *
 open_bulletin_board(OchushaApplication *application,
 		    OchushaBulletinBoard *board, IconLabel *tab_label)
@@ -356,6 +439,12 @@ open_bulletin_board(OchushaApplication *application,
 			       GTK_SIGNAL_FUNC(refresh_thread_button_cb),
 			       application,
 			       0);
+      gtk_toolbar_insert_stock(toolbar, GTK_STOCK_FIND,
+			       _("Start text search on current thread"),
+			       "start_text_search_on_thread",
+			       GTK_SIGNAL_FUNC(start_thread_search_button_cb),
+			       application,
+			       0);
       gtk_toolbar_insert_stock(toolbar, GTK_STOCK_JUMP_TO,
 			       _("Jump to new comer response"),
 			       "jump_to_new_comer_response",
@@ -399,6 +488,12 @@ open_bulletin_board(OchushaApplication *application,
       g_signal_connect(G_OBJECT(paned_notebook), "page_switched",
 		       G_CALLBACK(page_switched_cb), application);
 
+      threadlist_view_set_enable_native_search(THREADLIST_VIEW(view), FALSE);
+      threadlist_view_set_title_match_func(THREADLIST_VIEW(view),
+			(ThreadlistViewTitleMatchFunc *)title_match_func);
+
+      g_signal_connect(G_OBJECT(view), "interactive_search",
+		       G_CALLBACK(interactive_search_cb), application);
 
       job_args->application = application;
       job_args->board = board;
@@ -452,7 +547,7 @@ typedef struct _OpenThreadJobArgs
 void
 ochusha_open_thread(OchushaApplication *application,
 		    OchushaBulletinBoard *board, OchushaBBSThread *thread,
-		    char *dat_filename, int res_num, gboolean in_tab)
+		    const char *dat_filename, int res_num, gboolean in_tab)
 {
   WorkerJob *job;
   OpenThreadJobArgs *args;
@@ -484,7 +579,7 @@ ochusha_open_thread(OchushaApplication *application,
   args->application = application;
   args->board = board;
   args->thread = thread;
-  args->dat_filename = dat_filename;
+  args->dat_filename = g_strdup(dat_filename);
   args->res_num = res_num;
   args->in_tab = in_tab;
   commit_job(job);
@@ -497,12 +592,12 @@ open_thread_dat(WorkerThread *employee, gpointer job_args)
   OpenThreadJobArgs *args = (OpenThreadJobArgs *)job_args;
   OchushaApplication *application = args->application;
   OchushaBulletinBoard *board = args->board;
+  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board);
 
   THREAD_LIST_LOCK
     {
       PanedNotebook *board_view;
       OchushaBBSThread *thread;
-      BulletinBoardGUIInfo *info = (BulletinBoardGUIInfo *)board->user_data;
       if (info->is_busy)
 	THREAD_LIST_COND_WAIT;
 
@@ -526,8 +621,8 @@ open_thread_dat(WorkerThread *employee, gpointer job_args)
     }
   THREAD_LIST_UNLOCK;
 
-  free(args->dat_filename);
-  free(args);
+  g_free(args->dat_filename);
+  g_free(args);
 }
 
 
@@ -541,6 +636,7 @@ refresh_threadlist_view(OchushaApplication *application,
   WorkerJob *job;
   ThreadlistJobArgs *job_args;
   IconLabel *tab_label;
+  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board);
 
   g_return_if_fail(board != NULL && application->contents_window != NULL);
   contents_window = PANED_NOTEBOOK(application->contents_window);
@@ -555,7 +651,6 @@ refresh_threadlist_view(OchushaApplication *application,
 
   THREAD_LIST_LOCK
     {
-      BulletinBoardGUIInfo *info = (BulletinBoardGUIInfo *)board->user_data;
       OchushaAsyncBuffer *old_buffer;
       if (info->is_busy)
 	{
@@ -629,6 +724,7 @@ redraw_threadlist_view(OchushaApplication *application,
   WorkerJob *job;
   ThreadlistJobArgs *job_args;
   IconLabel *tab_label;
+  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board);
 
   g_return_if_fail(board != NULL && application->contents_window != NULL);
   contents_window = PANED_NOTEBOOK(application->contents_window);
@@ -645,7 +741,6 @@ redraw_threadlist_view(OchushaApplication *application,
 
   THREAD_LIST_LOCK
     {
-      BulletinBoardGUIInfo *info = (BulletinBoardGUIInfo *)board->user_data;
 
       while (info->is_busy)
 	THREAD_LIST_COND_WAIT;
@@ -694,6 +789,9 @@ typedef struct _EachThreadArgs
 {
   OchushaApplication *application;
   ThreadlistView *view;
+  int number_of_threads;
+
+  OchushaBBSThread *last_thread;
 
   ThreadlistFilter *filter;
   gboolean enable_filter;
@@ -708,6 +806,7 @@ each_thread_cb(OchushaBBSThread *thread, gpointer user_data)
   ThreadlistView *view = args->view;
   BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
   gboolean filter_result;
+
 #if DEBUG_GUI_MOST
   gchar *title = convert_string(utf8_to_native, thread->title, -1);
   fprintf(stderr, "each_thread_cb: %s <%d>\n",
@@ -715,32 +814,29 @@ each_thread_cb(OchushaBBSThread *thread, gpointer user_data)
   free(title);
 #endif
 
-  view->number_of_threads++;
+  args->number_of_threads++;
 
   filter_result = threadlist_filter_check_thread(args->filter,
 						 args->application, thread);
 
   if (filter_result || !args->enable_filter)
     {
-      if (args->enable_filter || !filter_result)
-	info->info.view_flags &= ~BBS_THREAD_HIGHLIGHT;
-      else
-	info->info.view_flags |= BBS_THREAD_HIGHLIGHT;
+      if (info->view_rank == 0)
+	info->view_flags |= BBS_THREAD_NEW;
+      else if (args->increment_ignored && !args->application->conf.offline)
+	info->view_flags &= ~BBS_THREAD_NEW;
+
       gdk_threads_enter();
-      threadlist_view_append_thread(view, thread, view->number_of_threads);
+      threadlist_view_append_thread(view, thread,
+				    info->view_rank, args->number_of_threads);
+      update_threadlist_entry_style(args->application, view, thread);
       gdk_threads_leave();
-      if (!args->application->conf.offline && info != NULL
-	  && args->increment_ignored)
+      args->last_thread = thread;
+      if (!args->application->conf.offline && args->increment_ignored)
 	info->view_ignored++;
     }
-  else
-    {
-      if (info != NULL)
-	info->info.threadlist_view = NULL;
-    }
 
-  if (!args->application->conf.offline && info != NULL)
-    info->info.view_rank = view->number_of_threads;
+  info->view_rank = args->number_of_threads;
 
   return TRUE;
 }
@@ -752,13 +848,15 @@ show_threadlist(WorkerThread *employee, gpointer args)
   ThreadlistJobArgs *job_args = (ThreadlistJobArgs *)args;
   OchushaApplication *application = job_args->application;
   OchushaBulletinBoard *board = job_args->board;
-  BulletinBoardGUIInfo *info = (BulletinBoardGUIInfo *)board->user_data;
+  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board);
   ThreadlistView *view = job_args->view;
   OchushaAsyncBuffer *buffer;
   EachThreadArgs cb_args =
     {
       application,
       view,
+      0,			/* number_of_threads */
+      NULL,			/* OchushaBBSThread *last_thread */
       &info->filter,
       info->enable_filter,
       job_args->increment_ignored
@@ -791,8 +889,7 @@ show_threadlist(WorkerThread *employee, gpointer args)
     }
 
   gdk_threads_enter();
-  if (!threadlist_view_open(view,
-			    (ThreadlistInfoNewFunc *)ensure_bbs_thread_info))
+  if (!threadlist_view_open(view))
     {
       gdk_threads_leave();
 #if DEBUG_GUI_MOST
@@ -811,7 +908,7 @@ show_threadlist(WorkerThread *employee, gpointer args)
       free(name);
 #endif
       gdk_threads_enter();
-      threadlist_view_close(view);
+      threadlist_view_close(view, NULL);
       gdk_threads_leave();
       goto finish_job;
     }
@@ -821,7 +918,7 @@ show_threadlist(WorkerThread *employee, gpointer args)
 #endif
 
   gdk_threads_enter();
-  threadlist_view_close(view);
+  threadlist_view_close(view, cb_args.last_thread);
   gdk_threads_leave();
 
  finish_job:
@@ -872,7 +969,7 @@ show_threadlist(WorkerThread *employee, gpointer args)
 OchushaAsyncBuffer *
 snatch_subject_txt_buffer_for_board(OchushaBulletinBoard *board)
 {
-  BulletinBoardGUIInfo *info = (BulletinBoardGUIInfo *)board->user_data;
+  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board);
   OchushaAsyncBuffer *buffer;
 
   THREAD_LIST_LOCK
@@ -891,18 +988,47 @@ void
 update_threadlist_entry_style(OchushaApplication *application,
 			      ThreadlistView *view, OchushaBBSThread *thread)
 {
-  BulletinBoardGUIInfo *board_info
-    = (BulletinBoardGUIInfo *)thread->board->user_data;
-  BBSThreadGUIInfo *thread_info = (BBSThreadGUIInfo *)thread->user_data;
+  BulletinBoardGUIInfo *board_info = ensure_bulletin_board_info(thread->board);
+  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
+  ThreadlistEntryVisualParameter visual_parameter =
+    {
+      0,	/* flags */
+      0,	/* reserved */
+      0,	/* foreground color */
+      0		/* background color */
+    };
+
+  gboolean exist_new_responses
+    = (thread->number_of_responses_on_server
+       > thread->number_of_responses_read);
+
+  /* flags */
+  if (info->view_flags & BBS_THREAD_HIDDEN)
+    visual_parameter.flags |= THREADLIST_ENTRY_FLAG_STRIKE;
+  if (info->view_flags & BBS_THREAD_FAVORITE)
+    visual_parameter.flags |= THREADLIST_ENTRY_FLAG_MARK;
+
+  /* foreground */
+  if (info->view_flags & BBS_THREAD_HIDDEN)
+    visual_parameter.foreground_color = LIST_ENTRY_FOREGROUND_COLOR_GRAY75;
+  else if (exist_new_responses)
+    {
+      if (info->view_flags & BBS_THREAD_FAVORITE)
+	visual_parameter.foreground_color = LIST_ENTRY_FOREGROUND_COLOR_RED;
+    }
+
+  /* background */
+  if (info->view_flags & BBS_THREAD_NEW)
+    visual_parameter.background_color = LIST_ENTRY_BACKGROUND_COLOR_PINK;
+  else if (exist_new_responses
+	   && threadlist_filter_check_thread(&board_info->filter,
+					     application,
+					     thread))
+    visual_parameter.background_color
+      = LIST_ENTRY_BACKGROUND_COLOR_LAVENDER;
 
-  if (board_info->enable_filter
-      || !threadlist_filter_check_thread(&board_info->filter, application,
-					 thread))
-    thread_info->info.view_flags &= ~BBS_THREAD_HIGHLIGHT;
-  else
-    thread_info->info.view_flags |= BBS_THREAD_HIGHLIGHT;
-    
   threadlist_view_update_thread(view, thread);
+  threadlist_view_update_thread_visual(view, thread, &visual_parameter);
 }
 
 
@@ -925,8 +1051,8 @@ static void
 toggle_mark_cb(ThreadlistView *view, OchushaBBSThread *thread,
 	       OchushaApplication *application)
 {
-  BBSThreadGUIInfo *info = (BBSThreadGUIInfo *)thread->user_data;
-  info->info.view_flags ^= BBS_THREAD_FAVORITE;
+  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
+  info->view_flags ^= BBS_THREAD_FAVORITE;
   update_threadlist_entry_style(application, view, thread);
 }
 
@@ -935,8 +1061,8 @@ static void
 toggle_hide_cb(ThreadlistView *view, OchushaBBSThread *thread,
 	       OchushaApplication *application)
 {
-  BBSThreadGUIInfo *info = (BBSThreadGUIInfo *)thread->user_data;
-  info->info.view_flags ^= BBS_THREAD_HIDDEN;
+  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
+  info->view_flags ^= BBS_THREAD_HIDDEN;
   update_threadlist_entry_style(application, view, thread);
 }
 
@@ -945,11 +1071,11 @@ static void
 mark_thread_cb(ThreadlistView *view, OchushaBBSThread *thread, gboolean do_mark,
 	       OchushaApplication *application)
 {
-  BBSThreadGUIInfo *info = (BBSThreadGUIInfo *)thread->user_data;
+  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
   if (do_mark)
-    info->info.view_flags |= BBS_THREAD_FAVORITE;
+    info->view_flags |= BBS_THREAD_FAVORITE;
   else
-    info->info.view_flags &= ~BBS_THREAD_FAVORITE;
+    info->view_flags &= ~BBS_THREAD_FAVORITE;
   update_threadlist_entry_style(application, view, thread);
 }
 
@@ -1080,7 +1206,7 @@ item_view_required_cb(PanedNotebook *paned_notebook,
 static void
 free_dat_buffer(OchushaBBSThread *thread, gpointer unused)
 {
-  BBSThreadGUIInfo *info = (BBSThreadGUIInfo *)thread->user_data;
+  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
 
 #if DEBUG_ASYNC_BUFFER_MOST
   fprintf(stderr, "free_dat_buffer: unref OchushaAsyncBuffer(0x%x)\n",
@@ -1101,8 +1227,13 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook, OchushaBBSThread *threa
 			  GtkWidget *item_view,
 			  OchushaApplication *application)
 {
-  BBSThreadGUIInfo *info = (BBSThreadGUIInfo *)thread->user_data;
-  OchushaAsyncBuffer *buffer = info->dat_buffer;
+  BBSThreadGUIInfo *info;
+  OchushaAsyncBuffer *buffer;
+
+  g_return_if_fail(GTK_IS_WIDGET(item_view));
+
+  info = ensure_bbs_thread_info(thread);
+  buffer = info->dat_buffer;
 
   /*
    * XXX: Ǥinfo->bufferοǤthread safetyݤǤƤʤ
@@ -1167,6 +1298,9 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook, OchushaBBSThread *threa
 
   ochusha_async_buffer_resume_all();
 #endif
+
+  if (application->last_search_target_widget == item_view)
+    application->last_search_target_widget = NULL;
 }
 
 
@@ -1174,7 +1308,7 @@ static void
 item_view_closed_cb(PanedNotebook *paned_notebook, OchushaBBSThread *thread,
 		    GtkWidget *item_view, OchushaApplication *application)
 {
-  BBSThreadGUIInfo *info = (BBSThreadGUIInfo *)thread->user_data;
+  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
   /* ΥåɽĤ줿 */
 #if DEBUG_GUI_MOST
   fprintf(stderr, "closed: thread=0x%x, item_view=0x%x\n",
@@ -1214,7 +1348,7 @@ page_switched_cb(PanedNotebook *paned_notebook,
       bbs_thread_view_place_cursor_onscreen(BBS_THREAD_VIEW(previous_page));
       if (thread != NULL)
 	{
-	  info = (BBSThreadGUIInfo *)thread->user_data;
+	  info = ensure_bbs_thread_info(thread);
 	  info->next_mark = NULL;
 	}
     }
@@ -1240,7 +1374,7 @@ page_switched_cb(PanedNotebook *paned_notebook,
   if (threadlist_view == NULL || thread == NULL)
     return;
 
-  info = (BBSThreadGUIInfo *)thread->user_data;
+  info = ensure_bbs_thread_info(thread);
   if (info->next_mark != NULL)
     bbs_thread_view_scroll_to_mark(BBS_THREAD_VIEW(new_page), info->next_mark);
 
@@ -1261,13 +1395,17 @@ get_current_thread(OchushaApplication *application,
   paned_notebook = PANED_NOTEBOOK(paned_notebook_get_current_item_view(paned_notebook));
   if (paned_notebook == NULL)
     {
-      *view = NULL;
-      *thread = NULL;
+      if (view != NULL)
+	*view = NULL;
+      if (thread != NULL)
+	*thread = NULL;
       return;
     }
 
-  *view = paned_notebook_get_current_item_view(paned_notebook);
-  *thread = (OchushaBBSThread *)paned_notebook_get_current_item(paned_notebook);
+  if (view != NULL)
+    *view = paned_notebook_get_current_item_view(paned_notebook);
+  if (thread != NULL)
+    *thread = (OchushaBBSThread *)paned_notebook_get_current_item(paned_notebook);
   if (tab_label != NULL)
     *tab_label = (IconLabel *)paned_notebook_get_tab_label(paned_notebook,
 							   *thread);
@@ -1346,6 +1484,124 @@ refresh_thread_button_cb(GtkWidget *widget, OchushaApplication *application)
 }
 
 
+static void
+text_search_window_response_cb(TextSearchWindow *window, gint response_id,
+			       OchushaApplication *application)
+{
+  switch (response_id)
+    {
+    case GTK_RESPONSE_DELETE_EVENT:
+      application->thread_search_window = NULL;
+      /* fall through */
+
+    case GTK_RESPONSE_CANCEL:
+      /* XXX: ɥξȥϿ٤ */
+      gtk_widget_hide(GTK_WIDGET(window));
+      if (application->last_search_target_widget != NULL)
+	{
+	  bbs_thread_view_invalidate_search_result(BBS_THREAD_VIEW(application->last_search_target_widget));
+	  application->last_search_target_widget = NULL;
+	}
+      return;
+    }
+
+  fprintf(stderr, "text_search_window_response_cb: unknown response(%d)\n",
+	  response_id);
+}
+
+
+static gboolean
+text_search_window_query_changed_cb(TextSearchWindow *window, const gchar *key,
+				    TextSearchDirection direction,
+				    gboolean enable_wrap, gboolean match_case,
+				    gboolean use_regexp,
+				    OchushaApplication *application)
+{
+  GtkWidget *widget;
+  get_current_thread(application, &widget, NULL, NULL);
+
+  if (application->last_search_target_widget != widget)
+    {
+      if (IS_BBS_THREAD_VIEW(application->last_search_target_widget))
+	bbs_thread_view_invalidate_search_result(BBS_THREAD_VIEW(application->last_search_target_widget));
+      application->last_search_target_widget = widget;
+    }
+
+  if (widget == NULL)
+    return FALSE;
+
+  g_return_val_if_fail(IS_BBS_THREAD_VIEW(widget), FALSE);
+  return bbs_thread_view_find(
+		BBS_THREAD_VIEW(application->last_search_target_widget),
+		key, direction, enable_wrap, match_case, use_regexp);
+}
+
+
+static gboolean
+text_search_window_find_next_cb(TextSearchWindow *window, const gchar *key,
+				TextSearchDirection direction,
+				gboolean enable_wrap, gboolean match_case,
+				gboolean use_regexp,
+				OchushaApplication *application)
+{
+  GtkWidget *widget;
+
+  get_current_thread(application, &widget, NULL, NULL);
+
+  if (application->last_search_target_widget == widget)
+    {
+      if (widget == NULL)
+	return FALSE;
+      return bbs_thread_view_find_next(BBS_THREAD_VIEW(widget));
+    }
+
+  if (application->last_search_target_widget != NULL)
+    bbs_thread_view_invalidate_search_result(BBS_THREAD_VIEW(application->last_search_target_widget));
+  application->last_search_target_widget = widget;
+
+  if (widget == NULL)
+    return FALSE;
+  return bbs_thread_view_find(
+		BBS_THREAD_VIEW(application->last_search_target_widget),
+		key, direction, enable_wrap, match_case, use_regexp);
+}
+
+
+static void
+setup_thread_search_window(OchushaApplication *application)
+{
+  application->thread_search_window = text_search_window_new();
+  text_search_window_set_enable_incremental_search(
+			TEXT_SEARCH_WINDOW(application->thread_search_window),
+			TRUE);
+  g_signal_connect(application->thread_search_window, "response",
+		   G_CALLBACK(text_search_window_response_cb),
+		   application);
+  g_signal_connect(application->thread_search_window, "query_changed",
+		   G_CALLBACK(text_search_window_query_changed_cb),
+		   application);
+  g_signal_connect(application->thread_search_window, "find_next",
+		   G_CALLBACK(text_search_window_find_next_cb),
+		   application);
+  gtk_window_set_title(GTK_WINDOW(application->thread_search_window),
+		       _("Find in the current thread"));
+  gtk_window_set_transient_for(GTK_WINDOW(application->thread_search_window),
+			       application->top_level);
+}
+
+
+static void
+start_thread_search_button_cb(GtkWidget *widget,
+			      OchushaApplication *application)
+{
+  if (application->thread_search_window == NULL)
+    setup_thread_search_window(application);
+
+  gtk_widget_show(application->thread_search_window);
+  gtk_widget_grab_focus(application->thread_search_window);
+}
+
+
 void
 jump_to_new_comer_response_of_current_thread(OchushaApplication *application)
 {
