@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.h,v 1.1.1.1 2003/05/10 16:34:25 fuyu Exp $
+ * $Id: bulletin_board_ui.h,v 1.5 2003/05/16 00:24:59 fuyu Exp $
  */
 
 #ifndef _BULLETIN_BOARD_UI_H_
@@ -49,8 +49,6 @@ extern "C" {
 /* BulletinBoardInfoΥ֥饹Ū˻Ȥ */
 typedef struct _BulletinBoardGUIInfo
 {
-  BulletinBoardInfo info;
-
   OchushaAsyncBuffer *subject_txt_buffer;
 
   GtkWidget *properties_dialog;
@@ -64,6 +62,24 @@ typedef struct _BulletinBoardGUIInfo
 } BulletinBoardGUIInfo;
 
 
+typedef struct _ThreadlistSearchQueryData
+{
+  gchar *key;
+  gint direction;
+  ThreadlistView *last_view;
+
+#if ENABLE_REGEXP
+  regex_t regexp;
+  gboolean regexp_available;
+#endif
+
+  gboolean enable_wrap;
+  gboolean match_case;
+  gboolean use_regexp;
+} ThreadlistSearchQueryData;
+
+
+void prepare_board_ui_initialization(OchushaApplication *application);
 void initialize_board_ui(OchushaApplication *application);
 
 BulletinBoardGUIInfo *ensure_bulletin_board_info(OchushaBulletinBoard *board);
@@ -75,7 +91,8 @@ void finalize_bulletin_board_view(OchushaBulletinBoard *board,
 
 void ochusha_open_thread(OchushaApplication *application,
 			 OchushaBulletinBoard *board, OchushaBBSThread *thread,
-			 char *dat_filename, int res_num, gboolean in_tab);
+			 const char *dat_filename, int res_num,
+			 gboolean in_tab);
 
 void refresh_threadlist_view(OchushaApplication *application,
 			     OchushaBulletinBoard *board);
