@@ -51,11 +51,13 @@
 
 /* VOID:INT (marshal.list:2) */
 
-/* VOID:POINTER (marshal.list:3) */
+/* VOID:OBJECT (marshal.list:3) */
 
-/* VOID:VOID (marshal.list:4) */
+/* VOID:POINTER (marshal.list:4) */
 
-/* VOID:ENUM,INT (marshal.list:5) */
+/* VOID:VOID (marshal.list:5) */
+
+/* VOID:ENUM,INT (marshal.list:6) */
 void
 ochusha_marshal_VOID__ENUM_INT (GClosure     *closure,
                                 GValue       *return_value,
@@ -92,7 +94,7 @@ ochusha_marshal_VOID__ENUM_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,OBJECT (marshal.list:6) */
+/* VOID:OBJECT,OBJECT (marshal.list:7) */
 void
 ochusha_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                      GValue       *return_value,
@@ -129,7 +131,7 @@ ochusha_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,BOOLEAN (marshal.list:7) */
+/* VOID:POINTER,BOOLEAN (marshal.list:8) */
 void
 ochusha_marshal_VOID__POINTER_BOOLEAN (GClosure     *closure,
                                        GValue       *return_value,
@@ -166,7 +168,7 @@ ochusha_marshal_VOID__POINTER_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,POINTER (marshal.list:8) */
+/* VOID:POINTER,POINTER (marshal.list:9) */
 void
 ochusha_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                        GValue       *return_value,
@@ -203,7 +205,7 @@ ochusha_marshal_VOID__POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,POINTER,STRING (marshal.list:9) */
+/* VOID:BOXED,POINTER,STRING (marshal.list:10) */
 void
 ochusha_marshal_VOID__BOXED_POINTER_STRING (GClosure     *closure,
                                             GValue       *return_value,
@@ -242,7 +244,7 @@ ochusha_marshal_VOID__BOXED_POINTER_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,POINTER,POINTER,POINTER (marshal.list:10) */
+/* VOID:POINTER,POINTER,POINTER,POINTER (marshal.list:11) */
 void
 ochusha_marshal_VOID__POINTER_POINTER_POINTER_POINTER (GClosure     *closure,
                                                        GValue       *return_value,
@@ -283,7 +285,7 @@ ochusha_marshal_VOID__POINTER_POINTER_POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* BOOLEAN:ENUM,INT (marshal.list:11) */
+/* BOOLEAN:ENUM,INT (marshal.list:12) */
 void
 ochusha_marshal_BOOLEAN__ENUM_INT (GClosure     *closure,
                                    GValue       *return_value,
@@ -324,7 +326,7 @@ ochusha_marshal_BOOLEAN__ENUM_INT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:POINTER (marshal.list:12) */
+/* BOOLEAN:POINTER (marshal.list:13) */
 void
 ochusha_marshal_BOOLEAN__POINTER (GClosure     *closure,
                                   GValue       *return_value,
@@ -363,7 +365,7 @@ ochusha_marshal_BOOLEAN__POINTER (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:BOXED,POINTER,STRING (marshal.list:13) */
+/* BOOLEAN:BOXED,POINTER,STRING (marshal.list:14) */
 void
 ochusha_marshal_BOOLEAN__BOXED_POINTER_STRING (GClosure     *closure,
                                                GValue       *return_value,
@@ -406,3 +408,50 @@ ochusha_marshal_BOOLEAN__BOXED_POINTER_STRING (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
+/* BOOLEAN:STRING,INT,BOOLEAN,BOOLEAN,BOOLEAN (marshal.list:15) */
+void
+ochusha_marshal_BOOLEAN__STRING_INT_BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
+                                                             GValue       *return_value,
+                                                             guint         n_param_values,
+                                                             const GValue *param_values,
+                                                             gpointer      invocation_hint,
+                                                             gpointer      marshal_data)
+{
+  typedef gboolean (*GMarshalFunc_BOOLEAN__STRING_INT_BOOLEAN_BOOLEAN_BOOLEAN) (gpointer     data1,
+                                                                                gpointer     arg_1,
+                                                                                gint         arg_2,
+                                                                                gboolean     arg_3,
+                                                                                gboolean     arg_4,
+                                                                                gboolean     arg_5,
+                                                                                gpointer     data2);
+  register GMarshalFunc_BOOLEAN__STRING_INT_BOOLEAN_BOOLEAN_BOOLEAN callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gboolean v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 6);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_BOOLEAN__STRING_INT_BOOLEAN_BOOLEAN_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_string (param_values + 1),
+                       g_marshal_value_peek_int (param_values + 2),
+                       g_marshal_value_peek_boolean (param_values + 3),
+                       g_marshal_value_peek_boolean (param_values + 4),
+                       g_marshal_value_peek_boolean (param_values + 5),
+                       data2);
+
+  g_value_set_boolean (return_value, v_return);
+}
+
