@@ -12,13 +12,16 @@ G_BEGIN_DECLS
 /* VOID:INT (marshal.list:2) */
 #define ochusha_marshal_VOID__INT	g_cclosure_marshal_VOID__INT
 
-/* VOID:POINTER (marshal.list:3) */
+/* VOID:OBJECT (marshal.list:3) */
+#define ochusha_marshal_VOID__OBJECT	g_cclosure_marshal_VOID__OBJECT
+
+/* VOID:POINTER (marshal.list:4) */
 #define ochusha_marshal_VOID__POINTER	g_cclosure_marshal_VOID__POINTER
 
-/* VOID:VOID (marshal.list:4) */
+/* VOID:VOID (marshal.list:5) */
 #define ochusha_marshal_VOID__VOID	g_cclosure_marshal_VOID__VOID
 
-/* VOID:ENUM,INT (marshal.list:5) */
+/* VOID:ENUM,INT (marshal.list:6) */
 extern void ochusha_marshal_VOID__ENUM_INT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -26,7 +29,7 @@ extern void ochusha_marshal_VOID__ENUM_INT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:OBJECT,OBJECT (marshal.list:6) */
+/* VOID:OBJECT,OBJECT (marshal.list:7) */
 extern void ochusha_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -34,7 +37,7 @@ extern void ochusha_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:POINTER,BOOLEAN (marshal.list:7) */
+/* VOID:POINTER,BOOLEAN (marshal.list:8) */
 extern void ochusha_marshal_VOID__POINTER_BOOLEAN (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -42,7 +45,7 @@ extern void ochusha_marshal_VOID__POINTER_BOOLEAN (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* VOID:POINTER,POINTER (marshal.list:8) */
+/* VOID:POINTER,POINTER (marshal.list:9) */
 extern void ochusha_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -50,7 +53,7 @@ extern void ochusha_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* VOID:BOXED,POINTER,STRING (marshal.list:9) */
+/* VOID:BOXED,POINTER,STRING (marshal.list:10) */
 extern void ochusha_marshal_VOID__BOXED_POINTER_STRING (GClosure     *closure,
                                                         GValue       *return_value,
                                                         guint         n_param_values,
@@ -58,7 +61,7 @@ extern void ochusha_marshal_VOID__BOXED_POINTER_STRING (GClosure     *closure,
                                                         gpointer      invocation_hint,
                                                         gpointer      marshal_data);
 
-/* VOID:POINTER,POINTER,POINTER,POINTER (marshal.list:10) */
+/* VOID:POINTER,POINTER,POINTER,POINTER (marshal.list:11) */
 extern void ochusha_marshal_VOID__POINTER_POINTER_POINTER_POINTER (GClosure     *closure,
                                                                    GValue       *return_value,
                                                                    guint         n_param_values,
@@ -66,7 +69,7 @@ extern void ochusha_marshal_VOID__POINTER_POINTER_POINTER_POINTER (GClosure
                                                                    gpointer      invocation_hint,
                                                                    gpointer      marshal_data);
 
-/* BOOLEAN:ENUM,INT (marshal.list:11) */
+/* BOOLEAN:ENUM,INT (marshal.list:12) */
 extern void ochusha_marshal_BOOLEAN__ENUM_INT (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -74,7 +77,7 @@ extern void ochusha_marshal_BOOLEAN__ENUM_INT (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* BOOLEAN:POINTER (marshal.list:12) */
+/* BOOLEAN:POINTER (marshal.list:13) */
 extern void ochusha_marshal_BOOLEAN__POINTER (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -82,7 +85,7 @@ extern void ochusha_marshal_BOOLEAN__POINTER (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* BOOLEAN:BOXED,POINTER,STRING (marshal.list:13) */
+/* BOOLEAN:BOXED,POINTER,STRING (marshal.list:14) */
 extern void ochusha_marshal_BOOLEAN__BOXED_POINTER_STRING (GClosure     *closure,
                                                            GValue       *return_value,
                                                            guint         n_param_values,
@@ -90,6 +93,14 @@ extern void ochusha_marshal_BOOLEAN__BOXED_POINTER_STRING (GClosure     *closure
                                                            gpointer      invocation_hint,
                                                            gpointer      marshal_data);
 
+/* BOOLEAN:STRING,INT,BOOLEAN,BOOLEAN,BOOLEAN (marshal.list:15) */
+extern void ochusha_marshal_BOOLEAN__STRING_INT_BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
+                                                                         GValue       *return_value,
+                                                                         guint         n_param_values,
+                                                                         const GValue *param_values,
+                                                                         gpointer      invocation_hint,
+                                                                         gpointer      marshal_data);
+
 G_END_DECLS
 
 #endif /* __ochusha_marshal_MARSHAL_H__ */
