@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.c,v 1.3 2003/05/11 18:20:37 fuyu Exp $
+ * $Id: ochusha_ui.c,v 1.8 2003/05/15 19:35:01 fuyu Exp $
  */
 
 #include "config.h"
@@ -66,7 +66,7 @@
 #include "ochusha48.xpm"
 #include "ochusha-banner.xpm"
 
-static void setup_icon_factory(void);
+static void setup_tab_label_icons(void);
 static void delete_event_cb(GtkWidget *widget, GdkEventAny *event,
 			    gpointer user_data);
 #if ENABLE_MAIN_TOOLBAR
@@ -114,6 +114,8 @@ OchushaNetworkCallbacks icon_indicator_funcs =
 
 
 static GtkItemFactory *main_menu_item_factory;
+static GdkPixbuf *tab_label_icons[7];
+static GQuark tab_label_icon_counter_id;
 
 
 static pthread_mutex_t main_ui_lock;
@@ -254,7 +256,7 @@ ochusha_init_application(OchushaApplication *application)
       abort();
     }
 
-  setup_icon_factory();
+  setup_tab_label_icons();
 
   banner_pixbuf
     = gdk_pixbuf_new_from_xpm_data((const char **)ochusha_banner_xpm);
@@ -344,6 +346,12 @@ ochusha_init_application(OchushaApplication *application)
   initialize_open_url_dialog(application);
 
 
+  /* GUIν */
+  prepare_boardlist_ui_initialization(application);
+  prepare_board_ui_initialization(application);
+  prepare_thread_ui_initialization(application);
+
+
   /* İν */
   initialize_boardlist(application);
 
@@ -409,19 +417,32 @@ ochusha_open_url(OchushaApplication *application, const gchar *url,
 				      &board, &thread_num, &thread,
 				      &from, &to, NULL))
     {
-      /* TODO: Υ֥֥饦ɽ */
       char command_line[COMMAND_LINE_BUFFER_SIZE];
       if (snprintf(command_line, COMMAND_LINE_BUFFER_SIZE,
 		   application->web_browser_template, url)
 	  < COMMAND_LINE_BUFFER_SIZE)
 	{
 	  GError *error = NULL;
-	  if (!g_spawn_command_line_async(command_line, &error))
+	  /*
+	   * MEMO: UTF-8Ǥ'~'E280BEˤʤäƤꡢ줬ޤޤ줿URL
+	   *       mozillaϤƤŪΥڡɽʤȤ
+	   *       餷(̤ǧδĶǤɽƤ褦ʵ)
+	   *       Τᤳnative localeλꤹencodingѴ롣
+	   *
+	   *       ȥѥå by Motonobu Ichimura <famao@momonga-linux.org>
+	   *
+	   * XXX:  UTF-8ʸnativeencodingѴǽȤϸ¤ʤ
+	   *       Τϥ顼åȤкפ롣
+	   */
+	  char *native_command_line
+	    = g_locale_from_utf8(command_line, -1, NULL, NULL, NULL);
+	  if (!g_spawn_command_line_async(native_command_line, &error))
 	    {
-	      /* 顼ɽ */
+	      /* TODO: 顼ɽ */
 	      fprintf(stderr, "spawning error: %s (%d)\n",
 		      error->message, error->code);
 	    }
+	  g_free(native_command_line);
 	}
       return;
     }
@@ -451,60 +472,51 @@ get_current_url(OchushaApplication *application)
 #endif
 
 
+#if DEBUG_GUI_MOST
+static void
+free_quark_data(gpointer data)
+{
+  fprintf(stderr, "freeing data(0x%x)\n", (int)data);
+  g_free(data);
+}
+#endif
+
+
 GtkWidget *
 get_tab_label(const gchar *label_text)
 {
-  GtkWidget *tab_label;
-  GtkWidget **icon_list;
-
-  icon_list = g_malloc(sizeof(GtkWidget *) * 7);
-  icon_list[TAB_LABEL_ICON_PROGRESS0]
-    = gtk_image_new_from_stock(OCHUSHA_NET_PROGRESS_ICON0, GTK_ICON_SIZE_MENU);
-  icon_list[TAB_LABEL_ICON_PROGRESS1]
-    = gtk_image_new_from_stock(OCHUSHA_NET_PROGRESS_ICON1, GTK_ICON_SIZE_MENU);
-  icon_list[TAB_LABEL_ICON_PROGRESS2]
-    = gtk_image_new_from_stock(OCHUSHA_NET_PROGRESS_ICON2, GTK_ICON_SIZE_MENU);
-  icon_list[TAB_LABEL_ICON_PROGRESS3]
-    = gtk_image_new_from_stock(OCHUSHA_NET_PROGRESS_ICON3, GTK_ICON_SIZE_MENU);
-  icon_list[TAB_LABEL_ICON_PREPARATION]
-    = gtk_image_new_from_stock(OCHUSHA_NET_PREPARATION_ICON, GTK_ICON_SIZE_MENU);
-  icon_list[TAB_LABEL_ICON_DONE]
-    = gtk_image_new_from_stock(OCHUSHA_NET_DONE_ICON, GTK_ICON_SIZE_MENU);
-  icon_list[TAB_LABEL_ICON_ERROR]
-    = gtk_image_new_from_stock(OCHUSHA_NET_ERROR_ICON, GTK_ICON_SIZE_MENU);
-  tab_label = icon_label_new(label_text, icon_list, 7, TAB_LABEL_ICON_PREPARATION);
-
-  return tab_label;
+  GtkWidget *label = icon_label_new(label_text,
+				tab_label_icons[TAB_LABEL_ICON_PREPARATION]);
+#if DEBUG_GUI_MOST
+  g_object_set_qdata_full(G_OBJECT(label), tab_label_icon_counter_id,
+			  g_new(int, 1), (GDestroyNotify)free_quark_data);
+#else
+  g_object_set_qdata_full(G_OBJECT(label), tab_label_icon_counter_id,
+			  g_new(int, 1), (GDestroyNotify)g_free);
+#endif
+  return label;
 }
 
 
 static void
-setup_icon_factory(void)
+setup_tab_label_icons(void)
 {
-  GtkIconFactory *icon_factory = gtk_icon_factory_new();
-  gtk_icon_factory_add(icon_factory,
-		       OCHUSHA_NET_PROGRESS_ICON0,
-		       gtk_icon_set_new_from_pixbuf(gdk_pixbuf_new_from_xpm_data((const char **)blue0_xpm)));
-  gtk_icon_factory_add(icon_factory,
-		       OCHUSHA_NET_PROGRESS_ICON1,
-		       gtk_icon_set_new_from_pixbuf(gdk_pixbuf_new_from_xpm_data((const char **)blue1_xpm)));
-  gtk_icon_factory_add(icon_factory,
-		       OCHUSHA_NET_PROGRESS_ICON2,
-		       gtk_icon_set_new_from_pixbuf(gdk_pixbuf_new_from_xpm_data((const char **)blue2_xpm)));
-  gtk_icon_factory_add(icon_factory,
-		       OCHUSHA_NET_PROGRESS_ICON3,
-		       gtk_icon_set_new_from_pixbuf(gdk_pixbuf_new_from_xpm_data((const char **)blue3_xpm)));
-  gtk_icon_factory_add(icon_factory,
-		       OCHUSHA_NET_PREPARATION_ICON,
-		       gtk_icon_set_new_from_pixbuf(gdk_pixbuf_new_from_xpm_data((const char **)orange_xpm)));
-  gtk_icon_factory_add(icon_factory,
-		       OCHUSHA_NET_DONE_ICON,
-		       gtk_icon_set_new_from_pixbuf(gdk_pixbuf_new_from_xpm_data((const char **)black_xpm)));
-  gtk_icon_factory_add(icon_factory,
-		       OCHUSHA_NET_ERROR_ICON,
-		       gtk_icon_set_new_from_pixbuf(gdk_pixbuf_new_from_xpm_data((const char **)red_xpm)));
-
-  gtk_icon_factory_add_default(icon_factory);
+  tab_label_icons[TAB_LABEL_ICON_PROGRESS0]
+    = gdk_pixbuf_new_from_xpm_data((const char **)blue0_xpm);
+  tab_label_icons[TAB_LABEL_ICON_PROGRESS1]
+    = gdk_pixbuf_new_from_xpm_data((const char **)blue1_xpm);
+  tab_label_icons[TAB_LABEL_ICON_PROGRESS2]
+    = gdk_pixbuf_new_from_xpm_data((const char **)blue2_xpm);
+  tab_label_icons[TAB_LABEL_ICON_PROGRESS3]
+    = gdk_pixbuf_new_from_xpm_data((const char **)blue3_xpm);
+  tab_label_icons[TAB_LABEL_ICON_PREPARATION]
+    = gdk_pixbuf_new_from_xpm_data((const char **)orange_xpm);
+  tab_label_icons[TAB_LABEL_ICON_DONE]
+    = gdk_pixbuf_new_from_xpm_data((const char **)black_xpm);
+  tab_label_icons[TAB_LABEL_ICON_ERROR]
+    = gdk_pixbuf_new_from_xpm_data((const char **)red_xpm);
+  tab_label_icon_counter_id
+    = g_quark_from_static_string("OchushaUI::icon-counter");
 }
 
 
@@ -857,14 +869,17 @@ static void
 access_started_cb(gpointer user_data)
 {
   IconLabel *icon_label;
+  int *counter;
 #if DEBUG_GUI
   fprintf(stderr, "access_started_cb\n");
 #endif
   icon_label = ICON_LABEL(user_data);
-  icon_label->counter = 0;
+  counter = g_object_get_qdata(G_OBJECT(icon_label),
+			       tab_label_icon_counter_id);
+  *counter = 0;
 
   gdk_threads_enter();
-  icon_label_alternate_icon(icon_label, TAB_LABEL_ICON_PREPARATION);
+  icon_label_set_icon(icon_label, tab_label_icons[TAB_LABEL_ICON_PREPARATION]);
   gdk_threads_leave();
 }
 
@@ -885,7 +900,7 @@ static void
 access_progressed_cb(gpointer user_data)
 {
   IconLabel *icon_label;
-
+  int *counter;
 #if DEBUG_GUI_MOST
   fprintf(stderr, "access_progressed_cb\n");
 #endif
@@ -896,11 +911,13 @@ access_progressed_cb(gpointer user_data)
   g_return_if_fail(IS_ICON_LABEL(user_data));
   icon_label = ICON_LABEL(user_data);
 
-  icon_label->counter = (icon_label->counter + 1) % 7;
+  counter = g_object_get_qdata(G_OBJECT(icon_label),
+			       tab_label_icon_counter_id);
+  *counter = (*counter + 1) % 7;
 
   gdk_threads_enter();
-  icon_label_alternate_icon(icon_label, 
-			    progress_icon_number[icon_label->counter]);
+  icon_label_set_icon(icon_label,
+		      tab_label_icons[progress_icon_number[*counter]]);
   gdk_threads_leave();
 }
 
@@ -921,7 +938,7 @@ access_completed_cb(gpointer user_data)
   icon_label = ICON_LABEL(user_data);
 
   gdk_threads_enter();
-  icon_label_alternate_icon(icon_label, TAB_LABEL_ICON_DONE);
+  icon_label_set_icon(icon_label, tab_label_icons[TAB_LABEL_ICON_DONE]);
   gdk_threads_leave();
 }
 
@@ -942,7 +959,7 @@ access_terminated_cb(gpointer user_data)
   icon_label = ICON_LABEL(user_data);
 
   gdk_threads_enter();
-  icon_label_alternate_icon(icon_label, TAB_LABEL_ICON_DONE);
+  icon_label_set_icon(icon_label, tab_label_icons[TAB_LABEL_ICON_DONE]);
   gdk_threads_leave();
 }
 
@@ -963,7 +980,7 @@ access_failed_cb(gpointer user_data)
   icon_label = ICON_LABEL(user_data);
 
   gdk_threads_enter();
-  icon_label_alternate_icon(icon_label, TAB_LABEL_ICON_ERROR);
+  icon_label_set_icon(icon_label, tab_label_icons[TAB_LABEL_ICON_ERROR]);
   gdk_threads_leave();
 }
 
