@@ -23,20 +23,42 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.h,v 1.3 2003/05/11 18:20:37 fuyu Exp $
+ * $Id: ochusha_ui.h,v 1.7 2003/05/19 04:15:24 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_UI_H_
 #define _OCHUSHA_UI_H_
 
+#include "config.h"
 #include "ochusha.h"
 #include "ochusha_private.h"
 #include "ochusha_bbs_table.h"
 
-
 #include <glib.h>
 #include <gtk/gtk.h>
 
+#if ENABLE_REGEXP
+# if HAVE_ONIGURUMA
+#  if HAVE_ONIG_ONIGPOSIX_H
+#    include <onig/onigposix.h>
+#  else
+#    error "Use of oniguruma without onig/onigposix.h header isn't considered."
+#  endif
+# else
+#  if HAVE_POSIX_REGEX
+#    if HAVE_SYS_TYPES_H
+#      include <sys/types.h>
+#    endif
+#    if HAVE_REGEX_H
+#      include <regex.h>
+#    else
+#      warning "I don't know suitable header file of your posix regex library."
+#    endif
+#  endif
+# endif
+#endif
+
+
 #define DEBUG_GUI		0
 #define DEBUG_GUI_MOST		0
 #define DEBUG_THREAD		0
@@ -71,15 +93,6 @@
 
 #define THREAD_VIEW_EXTRA_WIDTH			30
 
-
-#define OCHUSHA_NET_PROGRESS_ICON0		"Ochusha-Network-Progress0"
-#define OCHUSHA_NET_PROGRESS_ICON1		"Ochusha-Network-Progress1"
-#define OCHUSHA_NET_PROGRESS_ICON2		"Ochusha-Network-Progress2"
-#define OCHUSHA_NET_PROGRESS_ICON3		"Ochusha-Network-Progress3"
-#define OCHUSHA_NET_PREPARATION_ICON		"Ochusha-Network-Preparation"
-#define OCHUSHA_NET_DONE_ICON			"Ochusha-Network-Done"
-#define OCHUSHA_NET_ERROR_ICON			"Ochusha-Network-Error"
-
 #define OCHUSHA_MINIMIZED_ICON			"Ochusha-Minimized"
 
 #define TAB_LABEL_ICON_PROGRESS0		0
@@ -128,6 +141,13 @@ struct _OchushaApplication
   GtkClipboard *clipboard;
 
 
+  GtkWidget *threadlist_search_window;
+
+  /* ⸡Ϣ */
+  GtkWidget *thread_search_window;
+  GtkWidget *last_search_target_widget;
+
+
   /* İθǡ */
   OchushaBBSTable *table;
 
