@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: paned_notebook.c,v 1.1.1.1 2003/05/10 16:34:26 fuyu Exp $
+ * $Id: paned_notebook.c,v 1.2 2003/05/14 18:48:52 fuyu Exp $
  */
 
 #include "ochusha_private.h"
@@ -345,10 +345,6 @@ item_view_required_cb(PanedNotebook *paned_notebook, gpointer item,
 		      GtkWidget **tab_label)
 {
   /* TODO: 顼Ф */
-#if DEBUG_WIDGET
-  if (*item_view == NULL || *title == NULL)
-    fprintf(stderr, "Cannot open item.\n");
-#endif
 }
 
 
@@ -496,9 +492,6 @@ paned_notebook_destroy(GtkObject *object)
 {
   PanedNotebook *paned_notebook = (PanedNotebook *)object;
 
-  /* item_view_being_closedʥФޤΤǴ֤˹礦Τ
-   * ModNotebookγƥڡΥåȤƤΤ
-   */
   if (paned_notebook->items_shown != NULL)
     {
       g_slist_foreach(paned_notebook->items_shown,
@@ -509,18 +502,11 @@ paned_notebook_destroy(GtkObject *object)
       paned_notebook->items_shown = NULL;
     }
 
-  /*
-   * MEMO: ɥޥ͡ˤäƥץꥱ󤬽λ줿
-   *       ˸¤ꡢcontents_pane˴Ƥ
-   *       ȤȽΤǡad-hocɤ⡢֥Ȥ˿
-   *       ʤ褦ˤ롣
-   */
   if (paned_notebook->contents_pane != NULL)
-    g_object_unref(G_OBJECT(paned_notebook->contents_pane));
-#if DEBUG_WIDGET
-  else
-    fprintf(stderr, "paned_notebook.c: contents_pane seems to have already been destroyed.\n");
-#endif
+    {
+      g_object_unref(G_OBJECT(paned_notebook->contents_pane));
+      paned_notebook->contents_pane = NULL;
+    }
 
   if (GTK_OBJECT_CLASS(parent_class)->destroy)
     (*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
@@ -1122,9 +1108,6 @@ paned_notebook_open_item_view(PanedNotebook *paned_notebook, gpointer item,
 
   if (item_view != NULL)
     {
-#if DEBUG_WIDGET_MOST
-      fprintf(stderr, "Item already shown.\n");
-#endif
       if (in_tab)
 	return;
 
@@ -1153,9 +1136,6 @@ paned_notebook_open_item_view(PanedNotebook *paned_notebook, gpointer item,
   /* TODO: item_title == NULLξŬ˽٤ */
   if (item_view == NULL || item_title == NULL)
     {
-#if DEBUG_WIDGET
-      fprintf(stderr, "item_view or item_title is NULL\n");
-#endif
       return;	/* itemɽǽ */
     }
 
@@ -1185,11 +1165,8 @@ paned_notebook_open_item_view(PanedNotebook *paned_notebook, gpointer item,
 
   if (paned_notebook->number_of_items_shown++ == 0 || in_tab)
     {
-#if DEBUG_WIDGET_MOST
-      fprintf(stderr, "showing a page(%s) in tab.\n", item_title);
-#endif
-      mod_notebook_append_page_menu(paned_notebook->item_contents, item_view,
-				    tab_label, gtk_label_new(item_title));
+      mod_notebook_append_page(paned_notebook->item_contents, item_view,
+			       tab_label);
       if (paned_notebook->number_of_items_shown == 2
 	  && paned_notebook->tab_policy == GTK_POLICY_AUTOMATIC)
 	{
@@ -1200,25 +1177,19 @@ paned_notebook_open_item_view(PanedNotebook *paned_notebook, gpointer item,
       return;
     }
 
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "showing a page(%s) in current page.\n", item_title);
-#endif
   /* 򤵤Ƥڡ֤*/
   page = mod_notebook_get_current_page(paned_notebook->item_contents);
   if (page == -1)
     {
       /* */
-#if DEBUG_WIDGET
-      fprintf(stderr, "Internal Error: inconsistent number_of_items_shown.\n");
-#endif
-      mod_notebook_append_page_menu(paned_notebook->item_contents, item_view,
-				    tab_label, gtk_label_new(item_title));
+      mod_notebook_append_page(paned_notebook->item_contents, item_view,
+			       tab_label);
       return;
     }
 
   close_nth_page(paned_notebook, page);
-  mod_notebook_insert_page_menu(paned_notebook->item_contents, item_view,
-				tab_label, gtk_label_new(item_title), page);
+  mod_notebook_insert_page(paned_notebook->item_contents, item_view,
+			   tab_label, page);
   mod_notebook_set_current_page(paned_notebook->item_contents, page);
 }
 
@@ -1340,21 +1311,12 @@ switch_page_cb(ModNotebook *notebook, ModNotebookPage *page, guint page_num,
   GtkWidget *new_current_page = mod_notebook_get_nth_page(notebook, page_num);
   GtkWidget *previous_page = paned_notebook->current_item_view;
 
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "page #%d selected.\n", page_num);
-#endif
-
   if (previous_page == new_current_page)
     return;
 
   relation = find_item_view_relation(paned_notebook, new_current_page);
   if (relation == NULL)
-    {
-#if DEBUG_WIDGET
-      fprintf(stderr, "HELP! Relation missed.\n");
-#endif
-      return;
-    }
+    return;
 
   paned_notebook->current_item = relation->item;
   paned_notebook->current_item_view = new_current_page;
@@ -1383,9 +1345,6 @@ button_press_event_cb(ModNotebook *notebook, GdkEventButton *event,
     {
       gint page = mod_notebook_get_mouse_event_page_num(notebook,
 							(GdkEventAny*)event);
-#if DEBUG_WIDGET_MOST
-      fprintf(stderr, "PanedNotebook(button_press_event_cb): page=%d\n", page);
-#endif
       if (page != -1)
 	{
 	  close_nth_page(paned_notebook, page);
@@ -1417,10 +1376,6 @@ menu_changed_cb(GtkOptionMenu *option_menu, PanedNotebook *paned_notebook)
   ItemViewRelation *relation;
   GtkMenuItem *menu_item = paned_notebook->active_menu_item;
   gint page;
-#if DEBUG_WIDGET_MOST
-  gint history = gtk_option_menu_get_history(option_menu);
-  fprintf(stderr, "Option menu changed.  history=%d\n", history);
-#endif
 
   if (menu_item == NULL)
     return;
@@ -1442,9 +1397,6 @@ static void
 menu_item_activate_cb(GtkMenuItem *menu_item, PanedNotebook *paned_notebook)
 {
   paned_notebook->active_menu_item = menu_item;
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "Menu item activated.\n");
-#endif
 }
 
 
