@@ -23,14 +23,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: threadlist_filter.c,v 1.1.1.1 2003/05/10 16:34:26 fuyu Exp $
+ * $Id: threadlist_filter.c,v 1.3 2003/05/15 19:55:43 fuyu Exp $
  */
 
 #include "ochusha.h"
 #include "ochusha_ui.h"
 
 #include "bbs_thread_ui.h"
-#include "bulletin_board_ui.h"
+#include "threadlist_filter.h"
 
 #include "marshal.h"
 
@@ -1550,7 +1550,7 @@ threadlist_filter_check_thread(ThreadlistFilter *filter,
 			       OchushaApplication *application,
 			       OchushaBBSThread *thread)
 {
-  BBSThreadGUIInfo *thread_info = (BBSThreadGUIInfo *)thread->user_data;
+  BBSThreadGUIInfo *thread_info = ensure_bbs_thread_info(thread);
   const gchar *rules;
   int threshold;
   const gchar *cur_rule;
@@ -1584,8 +1584,7 @@ threadlist_filter_check_thread(ThreadlistFilter *filter,
 	  return FALSE;
 
 	case THREADLIST_FILTER_RULE_SHOW_FAVORITE:
-	  if (thread_info != NULL
-	      && thread_info->info.view_flags & BBS_THREAD_FAVORITE)
+	  if (thread_info->view_flags & BBS_THREAD_FAVORITE)
 	    {
 #if DEBUG_GUI_MOST
 	      fprintf(stderr, "TRUE: show favorite\n");
@@ -1595,8 +1594,7 @@ threadlist_filter_check_thread(ThreadlistFilter *filter,
 	  break;
       
 	case THREADLIST_FILTER_RULE_HIDE_NON_FAVORITE:
-	  if (thread_info != NULL
-	      && (thread_info->info.view_flags & BBS_THREAD_FAVORITE) == 0)
+	  if ((thread_info->view_flags & BBS_THREAD_FAVORITE) == 0)
 	    {
 #if DEBUG_GUI_MOST
 	      fprintf(stderr, "FALSE: hide non favorite\n");
@@ -1648,8 +1646,8 @@ threadlist_filter_check_thread(ThreadlistFilter *filter,
 	  break;
 
 	case THREADLIST_FILTER_RULE_SHOW_NEW_COMER_THREAD:
-	  if (thread_info != NULL
-	      && thread_info->info.view_rank == 0)
+	  if (thread_info->view_rank == 0
+	      || (thread_info->view_flags & BBS_THREAD_NEW) != 0)
 	    {
 #if DEBUG_GUI_MOST
 	      fprintf(stderr, "TRUE: show new comer thread.\n");
@@ -1692,7 +1690,7 @@ threadlist_filter_check_thread(ThreadlistFilter *filter,
 
 	case THREADLIST_FILTER_RULE_HIDE_MUCH_IGNORED_THREAD:
 	  if (threshold != 0
-	      && thread_info != NULL && thread_info->view_ignored > threshold)
+	      && thread_info->view_ignored > threshold)
 	    {
 #if DEBUG_GUI_MOST
 	      fprintf(stderr, "FALSE: hide thread that is too many times ignored.\n");
@@ -1702,8 +1700,7 @@ threadlist_filter_check_thread(ThreadlistFilter *filter,
 	  break;
 
 	case THREADLIST_FILTER_RULE_HIDE_HIDDEN_THREAD:
-	  if (thread_info != NULL
-	      && thread_info->info.view_flags & BBS_THREAD_HIDDEN)
+	  if ((thread_info->view_flags & BBS_THREAD_HIDDEN) != 0)
 	    {
 #if DEBUG_GUI_MOST
 	      fprintf(stderr, "FALSE: hide thread marked as hidden.\n");
