@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: threadlist_view.c,v 1.4 2003/05/11 23:04:36 fuyu Exp $
+ * $Id: threadlist_view.c,v 1.9 2003/05/17 08:14:47 fuyu Exp $
  */
 
 #include "config.h"
@@ -66,14 +66,6 @@ static gboolean threadlist_view_title_not_equals(GtkTreeModel *model,
 static gboolean threadlist_view_move_cursor(ThreadlistView *view,
 					    GtkMovementStep step,
 					    gint count);
-#if 0
-/*
- * TODO: Emacs^Säݤ񤤤¸ˤϡ롼Ǽ
- *       뤷ʤäݤʡ
- */
-static gboolean threadlist_view_search_title(ThreadlistView *view,
-					     int direction);
-#endif
 
 static void threadlist_view_do_on_selection(ThreadlistView *view,
 					    gint what_to_do);
@@ -150,6 +142,7 @@ enum {
   BACK_VIEW_SIGNAL,
 
   MOVE_CURSOR_SIGNAL,
+  INTERACTIVE_SEARCH_SIGNAL,
 
   DO_ON_SELECTION_SIGNAL,
   DO_ON_THREAD_AT_CURSOR_SIGNAL,
@@ -167,11 +160,6 @@ enum {
 };
 
 
-static GtkScrolledWindowClass *parent_class = NULL;
-static gint threadlist_view_signals[LAST_SIGNAL] =
-  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
-
-
 enum {
   TREE_VIEW_MOVE_CURSOR_SIGNAL,
   TREE_VIEW_UNSELECT_ALL_SIGNAL,
@@ -179,7 +167,6 @@ enum {
   TREE_VIEW_START_INTERACTIVE_SEARCH_SIGNAL,
   TREE_VIEW_LAST_SIGNAL
 };
-static gint tree_view_signals[TREE_VIEW_LAST_SIGNAL] = { 0, 0, 0 };
 
 
 enum {
@@ -191,17 +178,61 @@ enum {
 };
 
 
+struct _ThreadlistInfo
+{
+  GtkTreeIter iter;
+  ThreadlistView *threadlist_view;
+};
+
+
+static GtkScrolledWindowClass *parent_class = NULL;
+static gint threadlist_view_signals[LAST_SIGNAL] =
+  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+static gint tree_view_signals[TREE_VIEW_LAST_SIGNAL] = { 0, 0, 0 };
 static GtkItemFactory *popup_menu_item_factory = NULL;
+static GQuark list_entry_info_id;
+
+
+static const char *list_entry_foreground_color[16] =
+{
+  NULL,	/* åȤdefault */
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+};
 
 
-#if 0	/* TODO: 롼Ǽ뤷ʤäݤ */
-enum {
-  THREADLIST_SEARCH_FORWARD,
-  THREADLIST_SEARCH_BACKWARD,
-  THREADLIST_SEARCH_REGEXP_FORWARD,
-  THREADLIST_SEARCH_REGEXP_BACKWARD,
+static const char *list_entry_background_color[16] =
+{
+  NULL,	/* åȤdefault */
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
+  NULL,
 };
-#endif
 
 
 static void
@@ -353,27 +384,24 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
 		 G_TYPE_POINTER);
 
   threadlist_view_signals[MOVE_CURSOR_SIGNAL] =
-    g_signal_new("move_cursor",
+    g_signal_new("wrapped_move_cursor",
 		 G_TYPE_FROM_CLASS(klass),
 		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 		 G_STRUCT_OFFSET(ThreadlistViewClass, move_cursor),
 		 NULL, NULL,
 		 ochusha_marshal_BOOLEAN__ENUM_INT,
-		 G_TYPE_NONE, 2,
+		 G_TYPE_BOOLEAN, 2,
 		 GTK_TYPE_MOVEMENT_STEP,
 		 G_TYPE_INT);
-
-#if 0	/* TODO: 롼Ǽ뤷ʤäݤ */
-  threadlist_view_signals[SEARCH_TITLE_SIGNAL] =
-    g_signal_new("search_title",
+  threadlist_view_signals[INTERACTIVE_SEARCH_SIGNAL] =
+    g_signal_new("interactive_search",
 		 G_TYPE_FROM_CLASS(klass),
 		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
-		 G_STRUCT_OFFSET(ThreadlistViewClass, search_title),
+		 G_STRUCT_OFFSET(ThreadlistViewClass, interactive_search),
 		 NULL, NULL,
 		 ochusha_marshal_VOID__INT,
 		 G_TYPE_NONE, 1,
 		 G_TYPE_INT);
-#endif
 
   threadlist_view_signals[DO_ON_SELECTION_SIGNAL] =
     g_signal_new("do_on_selection",
@@ -424,27 +452,66 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
 			       "do_on_thread_at_cursor", 1,
 			       G_TYPE_INT, BACK_VIEW);
 
+  gtk_binding_entry_add_signal(binding_set, GDK_Up, 0,
+			       "wrapped_move_cursor", 2,
+			       G_TYPE_ENUM, GTK_MOVEMENT_DISPLAY_LINES,
+			       G_TYPE_INT, -1);
+  gtk_binding_entry_add_signal(binding_set, GDK_Down, 0,
+			       "wrapped_move_cursor", 2,
+			       G_TYPE_ENUM, GTK_MOVEMENT_DISPLAY_LINES,
+			       G_TYPE_INT, 1);
+
+  gtk_binding_entry_add_signal(binding_set, GDK_p, GDK_CONTROL_MASK,
+			       "wrapped_move_cursor", 2,
+			       G_TYPE_ENUM, GTK_MOVEMENT_DISPLAY_LINES,
+			       G_TYPE_INT, -1);
+  gtk_binding_entry_add_signal(binding_set, GDK_n, GDK_CONTROL_MASK,
+			       "wrapped_move_cursor", 2,
+			       G_TYPE_ENUM, GTK_MOVEMENT_DISPLAY_LINES,
+			       G_TYPE_INT, 1);
+
   gtk_binding_entry_add_signal(binding_set, GDK_v, GDK_CONTROL_MASK,
-			       "move_cursor", 2,
+			       "wrapped_move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_PAGES,
 			       G_TYPE_INT, 1);
   gtk_binding_entry_add_signal(binding_set, GDK_v, GDK_MOD1_MASK,
-			       "move_cursor", 2,
+			       "wrapped_move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_PAGES,
 			       G_TYPE_INT, -1);
   gtk_binding_entry_add_signal(binding_set, GDK_v, GDK_MOD4_MASK,
-			       "move_cursor", 2,
+			       "wrapped_move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_PAGES,
 			       G_TYPE_INT, -1);
 
-#if 0	/* TODO: 롼Ǽ뤷ʤäݤ */
+  gtk_binding_entry_add_signal(binding_set, GDK_f, GDK_CONTROL_MASK,
+			       "interactive_search", 1,
+			       G_TYPE_INT, THREADLIST_SEARCH_ACTION_FORWARD);
+
   gtk_binding_entry_add_signal(binding_set, GDK_s, GDK_CONTROL_MASK,
-			       "search_title", 1,
-			       G_TYPE_ENUM, THREADLIST_SEARCH_FORWARD);
+			       "interactive_search", 1,
+			       G_TYPE_INT, THREADLIST_SEARCH_ACTION_FORWARD);
   gtk_binding_entry_add_signal(binding_set, GDK_r, GDK_CONTROL_MASK,
-			       "search_title", 1,
-			       G_TYPE_ENUM, THREADLIST_SEARCH_BACKWARD);
-#endif
+			       "interactive_search", 1,
+			       G_TYPE_INT, THREADLIST_SEARCH_ACTION_BACKWARD);
+  gtk_binding_entry_add_signal(binding_set, GDK_S,
+			       GDK_CONTROL_MASK | GDK_SHIFT_MASK,
+			       "interactive_search", 1,
+			       G_TYPE_INT,
+			       THREADLIST_SEARCH_ACTION_REGEXP_FORWARD);
+  gtk_binding_entry_add_signal(binding_set, GDK_R,
+			       GDK_CONTROL_MASK | GDK_SHIFT_MASK,
+			       "interactive_search", 1,
+			       G_TYPE_INT,
+			       THREADLIST_SEARCH_ACTION_REGEXP_BACKWARD);
+
+  gtk_binding_entry_add_signal(binding_set, GDK_Escape, 0,
+			       "interactive_search", 1,
+			       G_TYPE_INT,
+			       THREADLIST_SEARCH_ACTION_TERMINATE);
+  gtk_binding_entry_add_signal(binding_set, GDK_g, GDK_CONTROL_MASK,
+			       "interactive_search", 1,
+			       G_TYPE_INT,
+			       THREADLIST_SEARCH_ACTION_TERMINATE);
 
   klass->open_view = NULL;
 
@@ -458,19 +525,36 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
   klass->back_view = NULL;
 
   klass->move_cursor = threadlist_view_move_cursor;
-#if 0	/* TODO: 롼Ǽ뤷ʤäݤ */
-  klass->search_title = threadlist_view_search_title;
-#endif
+  klass->interactive_search = NULL;
 
   klass->do_on_selection = threadlist_view_do_on_selection;
   klass->do_on_thread_at_cursor = threadlist_view_do_on_thread_at_cursor;
+
+  list_entry_info_id
+    = g_quark_from_static_string("ThreadlistView::ListEntryInfo");
+
+  /*
+   * XXX: ʲñʤǥեȤȤ
+   *      桼ˤ񤭴ǽˤ٤
+   */
+  list_entry_foreground_color[1] = g_strdup("blue");
+  list_entry_foreground_color[2] = g_strdup("red");
+  list_entry_foreground_color[3] = g_strdup("green");
+  list_entry_foreground_color[4] = g_strdup("gray50");
+  list_entry_foreground_color[5] = g_strdup("gray75");
+
+  list_entry_background_color[1] = g_strdup("lavender");
+  list_entry_background_color[2] = g_strdup("#ffe0e0");
+  list_entry_background_color[3] = g_strdup("DarkSeaGreen1");
+  list_entry_background_color[4] = g_strdup("gray75");
+  list_entry_background_color[5] = g_strdup("gray90");
 }
 
 
 typedef enum
 {
   THREADLIST_RANK = 0,
-  THREADLIST_MARK,
+  THREADLIST_VISUAL_PARAMETER,
   THREADLIST_TITLE,
   THREADLIST_N_RESPONSES,
   THREADLIST_N_RESPONSES_READ,
@@ -519,17 +603,13 @@ threadlist_view_init(ThreadlistView *view)
   g_signal_connect(G_OBJECT(view->view), "button_release_event",
 		   G_CALLBACK(tree_view_button_release_event_cb), view);
 
-  view->model = NULL;
-
-  view->number_of_threads = 0;
-
   renderer = gtk_cell_renderer_text_new();
   column = gtk_tree_view_column_new_with_attributes(_("Rank"), renderer,
 						    "text", THREADLIST_RANK,
 						    NULL);
   gtk_tree_view_column_set_cell_data_func(column, renderer,
 					  decorate_threadlist_entry,
-					  NULL, NULL);
+					  view, NULL);
   gtk_tree_view_append_column(view->view, column);
   view->rank_column = column;
 
@@ -537,11 +617,11 @@ threadlist_view_init(ThreadlistView *view)
   gtk_cell_renderer_toggle_set_radio(GTK_CELL_RENDERER_TOGGLE(renderer),
 				     FALSE);
   column = gtk_tree_view_column_new_with_attributes(_("Mark"), renderer,
-						    "active", THREADLIST_MARK,
-						    NULL);
+					"active", THREADLIST_VISUAL_PARAMETER,
+					NULL);
   gtk_tree_view_column_set_cell_data_func(column, renderer,
 					  active_set_cell_data_flags,
-					  NULL, NULL);
+					  view, NULL);
   gtk_tree_view_append_column(view->view, column);
   view->mark_column = column;
 
@@ -551,7 +631,7 @@ threadlist_view_init(ThreadlistView *view)
 						    THREADLIST_TITLE, NULL);
   gtk_tree_view_column_set_cell_data_func(column, renderer,
 					  decorate_threadlist_entry,
-					  NULL, NULL);
+					  view, NULL);
   gtk_tree_view_append_column(view->view, column);
   view->title_column = column;
 
@@ -562,7 +642,7 @@ threadlist_view_init(ThreadlistView *view)
 						    NULL);
   gtk_tree_view_column_set_cell_data_func(column, renderer,
 					  decorate_threadlist_entry,
-					  NULL, NULL);
+					  view, NULL);
   gtk_tree_view_append_column(view->view, column);
   view->n_res_column = column;
 
@@ -573,7 +653,7 @@ threadlist_view_init(ThreadlistView *view)
 						    NULL);
   gtk_tree_view_column_set_cell_data_func(column, renderer,
 					  decorate_threadlist_entry,
-					  NULL, NULL);
+					  view, NULL);
   gtk_tree_view_append_column(view->view, column);
   view->n_read_column = column;
 
@@ -584,15 +664,20 @@ threadlist_view_init(ThreadlistView *view)
 						    NULL);
   gtk_tree_view_column_set_cell_data_func(column, renderer,
 					  text_set_cell_data_rank_diff,
-					  NULL, NULL);
+					  view, NULL);
   gtk_tree_view_append_column(view->view, column);
   view->rank_diff_column = column;
 
+  view->model = NULL;
+  view->model_is_opened = FALSE;
+
   gtk_tree_view_set_search_column(view->view, THREADLIST_TITLE);
   gtk_tree_view_set_search_equal_func(view->view,
 				      threadlist_view_title_not_equals,
 				      NULL, NULL);
   gtk_tree_view_set_enable_search(view->view, TRUE);
+  view->use_native_search = TRUE;
+  view->being_searched = FALSE;
 }
 
 
@@ -622,16 +707,39 @@ threadlist_view_finalize(GObject *object)
 static void
 threadlist_view_destroy(GtkObject *object)
 {
+  ThreadlistView *view = THREADLIST_VIEW(object);
 #if DEBUG_WIDGET_MOST
   fprintf(stderr, "threadlist_view_destroy: object->ref_count=%d\n",
 	  G_OBJECT(object)->ref_count);
 #endif
 
+  if (view->last_path != NULL)
+    {
+      gtk_tree_path_free(view->last_path);
+      view->last_path = NULL;
+    }
+
   if (GTK_OBJECT_CLASS(parent_class)->destroy)
     (*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
 }
 
 
+static ThreadlistInfo *
+ensure_threadlist_info(OchushaBBSThread *thread)
+{
+  ThreadlistInfo *info = g_object_get_qdata(G_OBJECT(thread),
+					    list_entry_info_id);
+  if (info == NULL)
+    {
+      info = g_new0(ThreadlistInfo, 1);
+      g_object_set_qdata_full(G_OBJECT(thread), list_entry_info_id,
+			      info, (GDestroyNotify)g_free);
+    }
+
+  return info;
+}
+
+
 static gboolean
 threadlist_view_title_not_equals(GtkTreeModel *model, gint column,
 				 const gchar *key, GtkTreeIter *iter,
@@ -694,13 +802,278 @@ static gboolean
 threadlist_view_move_cursor(ThreadlistView *view, GtkMovementStep step,
 			    gint count)
 {
-  gboolean result;
+  gboolean result = FALSE;
+
+  /*
+   * use_native_searchǤʤ硢ġʤ饫ưѤ
+   * ʥФ
+   */
+#if DEBUG_WIDGET_MOST
+  fprintf(stderr, "view->being_searched=%d\n", view->being_searched);
+#endif
+  if (!view->use_native_search && view->being_searched
+      && step == GTK_MOVEMENT_DISPLAY_LINES)
+    {
+      g_signal_emit(G_OBJECT(view),
+		    threadlist_view_signals[INTERACTIVE_SEARCH_SIGNAL],
+		    0,
+		    (count == 1
+		     ? THREADLIST_SEARCH_ACTION_FORWARD
+		     : THREADLIST_SEARCH_ACTION_BACKWARD),
+		    &result);
+      return TRUE;
+    }
 
   g_signal_emit(G_OBJECT(view->view),
 		tree_view_signals[TREE_VIEW_MOVE_CURSOR_SIGNAL],
 		0,
 		step, count,
 		&result);
+
+  return result;
+}
+
+
+static void
+find_initial_path_helper(GtkTreeModel *model, GtkTreePath *path,
+			 GtkTreeIter *iter, gpointer data)
+{
+  GtkTreePath **initial_path = (GtkTreePath **)data;
+  if (*initial_path != NULL)
+    return;
+
+  *initial_path = gtk_tree_path_copy(path);
+  return;
+}
+
+
+static GtkTreePath *
+threadlist_view_get_current_path(ThreadlistView *view)
+{
+  GtkTreePath *initial_path = NULL;
+  GtkTreeIter iter;
+
+  gtk_tree_view_get_cursor(view->view, &initial_path, NULL);
+  if (initial_path != NULL)
+    return initial_path;
+
+  gtk_widget_grab_focus(GTK_WIDGET(view));
+  gtk_tree_view_get_cursor(view->view, &initial_path, NULL);
+  if (initial_path != NULL)
+    return initial_path;
+
+  gtk_tree_selection_selected_foreach(view->selection,
+				      find_initial_path_helper, &initial_path);
+  if (initial_path != NULL)
+    return initial_path;
+
+  if (!gtk_tree_model_get_iter_first(view->model, &iter))
+    return NULL;
+
+  return gtk_tree_model_get_path(view->model, &iter);
+}
+
+
+void
+threadlist_view_set_being_searched(ThreadlistView *view, gboolean being)
+{
+  g_return_if_fail(IS_THREADLIST_VIEW(view));
+
+  view->being_searched = (being ? 1 : 0);
+}
+
+
+gboolean
+threadlist_view_find_next(ThreadlistView *view,
+			  ThreadlistSearchDirection direction,
+			  gboolean enable_wrap, gboolean include_current_pos,
+			  gpointer search_data)
+{
+  GtkTreeSelection *selection;
+  GtkTreeIter iter;
+  GtkTreePath *initial_path;
+  GtkTreePath *path = NULL;
+  gboolean advance_iter = FALSE;
+
+  g_return_val_if_fail(IS_THREADLIST_VIEW(view), FALSE);
+  g_return_val_if_fail(search_data != NULL, FALSE);
+  g_return_val_if_fail(view->title_match_func != NULL
+		       && !view->use_native_search, FALSE);
+
+  view->being_searched = TRUE;
+
+  selection = view->selection;
+  initial_path = threadlist_view_get_current_path(view);
+  if (initial_path == NULL)
+    return FALSE;
+
+  gtk_tree_selection_unselect_all(selection);
+
+  if (!gtk_tree_model_get_iter(view->model, &iter, initial_path))
+    {
+#if DEBUG_WIDGET_MOST
+      fprintf(stderr, "Couldn't get iter.\n");
+#endif
+      goto no_match_found;
+    }
+
+  if (!include_current_pos)
+    advance_iter = TRUE;
+
+  while (TRUE)
+    {
+      const gchar *title;
+      if (advance_iter)
+	{
+	  if (direction == THREADLIST_SEARCH_DIRECTION_FORWARD)
+	    {
+	      if (!gtk_tree_model_iter_next(view->model, &iter))
+		{
+		  if (view->last_search_position == 1 && enable_wrap)
+		    {
+		      view->last_search_position = 0;
+		      if (!gtk_tree_model_get_iter_first(view->model, &iter))
+			{
+#if DEBUG_WIDGET_MOST
+			  fprintf(stderr, "Couldn't get first iter.\n");
+#endif
+			  goto no_match_found;
+			}
+		    }
+		  else
+		    {
+		      {
+			view->last_search_position = 1;
+			goto no_match_found;
+		      }
+		    }
+		}
+	    }
+	  else
+	    {
+	      if (path == NULL)
+		path = gtk_tree_path_copy(initial_path);
+
+	      if (path == NULL)
+		goto no_match_found;
+
+	      if (!gtk_tree_path_prev(path))
+		{
+		  if (view->last_search_position == -1 && enable_wrap)
+		    {
+		      view->last_search_position = 0;
+		      if (view->last_path == NULL)
+			{
+#if DEBUG_WIDGET_MOST
+			  fprintf(stderr, "Couldn't get last iter.\n");
+#endif
+			  goto no_match_found;
+			}
+		      if (path != NULL)
+			gtk_tree_path_free(path);
+		      path = gtk_tree_path_copy(view->last_path);
+		    }
+		  else
+		    {
+		      view->last_search_position = -1;
+		      goto no_match_found;
+		    }
+		}
+
+	      if (!gtk_tree_model_get_iter(view->model, &iter, path))
+		{
+#if DEBUG_WIDGET_MOST
+		  fprintf(stderr, "Couldn't get prev iter\n");
+#endif
+		  goto no_match_found;
+		}
+	    }
+	}
+
+      gtk_tree_model_get(view->model, &iter, THREADLIST_TITLE, &title, -1);
+      if (title == NULL)
+	{
+#if DEBUG_WIDGET_MOST
+	  fprintf(stderr, "title is NULL\n");
+#endif
+	  goto no_match_found;
+	}
+
+      if ((*view->title_match_func)(title, search_data))
+	{
+	  /* found */
+	  gtk_tree_path_free(initial_path);
+	  if (path != NULL)
+	    gtk_tree_path_free(path);
+	  path = gtk_tree_model_get_path(view->model, &iter);
+	  gtk_tree_view_scroll_to_cell(view->view, path, NULL,
+				       TRUE, 0.5, 0.0);
+	  gtk_tree_selection_select_iter(selection, &iter);
+	  gtk_tree_view_set_cursor(view->view, path, NULL, FALSE);
+	  gtk_tree_path_free(path);
+
+	  return TRUE;
+	}
+
+      advance_iter = TRUE;
+    }
+
+ no_match_found:
+  if (path != NULL)
+    gtk_tree_path_free(path);
+
+  gtk_tree_view_set_cursor(view->view, initial_path, NULL, FALSE);
+  gtk_tree_path_free(initial_path);
+
+  return FALSE;
+}
+
+
+void
+threadlist_view_set_enable_native_search(ThreadlistView *view,
+					 gboolean enable)
+{
+  g_return_if_fail(IS_THREADLIST_VIEW(view));
+
+  gtk_tree_view_set_enable_search(view->view, enable);
+  view->use_native_search = enable;
+}
+
+
+gboolean
+threadlist_view_get_enable_native_search(ThreadlistView *view)
+{
+  g_return_val_if_fail(IS_THREADLIST_VIEW(view), FALSE);
+
+  return gtk_tree_view_get_enable_search(view->view);
+}
+
+
+static gboolean
+threadlist_view_interactive_search(ThreadlistView *view,
+				   ThreadlistSearchAction search_action)
+{
+  gboolean result;
+
+  if (view->use_native_search)
+    {
+      gtk_widget_grab_focus(GTK_WIDGET(view->view));
+
+      g_signal_emit(G_OBJECT(view->view),
+		tree_view_signals[TREE_VIEW_START_INTERACTIVE_SEARCH_SIGNAL],
+		0,
+		&result);
+    }
+  else
+    {
+      view->being_searched = TRUE;
+      g_signal_emit(G_OBJECT(view),
+		    threadlist_view_signals[INTERACTIVE_SEARCH_SIGNAL],
+		    0,
+		    search_action,
+		    &result);
+    }
+
   return result;
 }
 
@@ -708,26 +1081,28 @@ threadlist_view_move_cursor(ThreadlistView *view, GtkMovementStep step,
 gboolean
 threadlist_view_start_interactive_search(ThreadlistView *view)
 {
-  gboolean result;
   g_return_val_if_fail(IS_THREADLIST_VIEW(view) && view->view != NULL, FALSE);
 
-  gtk_widget_grab_focus(GTK_WIDGET(view->view));
-
-  g_signal_emit(G_OBJECT(view->view),
-		tree_view_signals[TREE_VIEW_START_INTERACTIVE_SEARCH_SIGNAL],
-		0,
-		&result);
-
-  return result;
+  return threadlist_view_interactive_search(view,
+					THREADLIST_SEARCH_DIRECTION_FORWARD);
 }
 
 
-#if 0	/* TODO: 롼Ǽ뤷ʤäݤ */
-static gboolean
-threadlist_view_search_title(ThreadlistView *view, int direction)
+void
+threadlist_view_set_title_match_func(ThreadlistView *view,
+				     ThreadlistViewTitleMatchFunc *func)
 {
+  g_return_if_fail(IS_THREADLIST_VIEW(view));
+  view->title_match_func = func;
+}
+
+
+ThreadlistViewTitleMatchFunc *
+threadlist_view_get_title_match_func(ThreadlistView *view)
+{
+  g_return_val_if_fail(IS_THREADLIST_VIEW(view), NULL);
+  return view->title_match_func;
 }
-#endif
 
 
 typedef struct _DoOnSelectionArgs
@@ -1047,18 +1422,18 @@ tree_view_button_release_event_cb(GtkWidget *widget, GdkEventButton *event,
 
 static void
 set_thread_values(ThreadlistView *view, GtkListStore *store,
-		  int rank, OchushaBBSThread *thread)
+		  int last_rank, int new_rank, OchushaBBSThread *thread)
 {
-  ThreadlistInfo *info = thread->user_data;
+  ThreadlistInfo *info = ensure_threadlist_info(thread);
 
-  if (rank != RANK_UNSPECIFIED)
+  if (new_rank != RANK_UNSPECIFIED)
     {
       gint rank_diff;
-      if (rank == RANK_DAT_DROPPED)
+      if (new_rank == RANK_DAT_DROPPED)
 	rank_diff = RANK_NOT_CHANGED;
       else
-	rank_diff = (info->view_rank == 0
-		     ? RANK_NEW_COMER_MAGIC : info->view_rank - rank);
+	rank_diff = (last_rank == 0
+		     ? RANK_NEW_COMER_MAGIC : last_rank - new_rank);
       if (view->last_rank_diff != RANK_NEW_COMER_MAGIC
 	  && view->last_rank_diff != RANK_NOT_CHANGED
 	  && view->last_rank_diff == rank_diff
@@ -1068,8 +1443,8 @@ set_thread_values(ThreadlistView *view, GtkListStore *store,
 	view->last_rank_diff = rank_diff;
 
       gtk_list_store_set(store, &info->iter,
-		THREADLIST_RANK, rank,
-		THREADLIST_MARK, info->view_flags,
+		THREADLIST_RANK, new_rank,
+		THREADLIST_VISUAL_PARAMETER, 0,
 		THREADLIST_TITLE, thread->title,
 		THREADLIST_N_RESPONSES, thread->number_of_responses_on_server,
 		THREADLIST_N_RESPONSES_READ, thread->number_of_responses_read,
@@ -1081,7 +1456,6 @@ set_thread_values(ThreadlistView *view, GtkListStore *store,
     {
       gtk_list_store_set(store, &info->iter,
 		THREADLIST_TITLE, thread->title,
-		THREADLIST_MARK, info->view_flags,
 		THREADLIST_N_RESPONSES, thread->number_of_responses_on_server,
 		THREADLIST_N_RESPONSES_READ, thread->number_of_responses_read,
 		THREADLIST_DATA, thread,
@@ -1090,29 +1464,124 @@ set_thread_values(ThreadlistView *view, GtkListStore *store,
 }
 
 
+static guint
+threadlist_view_encode_visual_params(ThreadlistView *view,
+				     ThreadlistEntryVisualParameter *parameter)
+{
+  guint result = 0;
+
+  result |= (parameter->flags & 3);
+  result |= (parameter->foreground_color & 0xf)
+    << (THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH);
+  result |= (parameter->background_color & 0xf)
+    << (THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH
+	+ THREADLIST_ENTRY_VISUAL_FOREGROUND_COLOR_WIDTH);
+#if DEBUG_WIDGET_MOST
+  fprintf(stderr, "parameter{ flags=%d, foreground_color=%d, background_color=%d}, result=%d\n", parameter->flags, parameter->foreground_color, parameter->background_color, result);
+#endif
+
+  return result;
+}
+
+
+static guint
+threadlist_view_decode_visual_param_flags(ThreadlistView *view,
+					  guint parameter)
+{
+  return parameter & 3;
+}
+
+
+static guint
+threadlist_view_decode_visual_param_foreground_color(ThreadlistView *view,
+						     guint parameter)
+{
+  return (parameter >> THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH) & 0xf;
+}
+
+
+static guint
+threadlist_view_decode_visual_param_background_color(ThreadlistView *view,
+						     guint parameter)
+{
+  return (parameter
+	  >> (THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH
+	      + THREADLIST_ENTRY_VISUAL_FOREGROUND_COLOR_WIDTH)) & 0xf;
+}
+
+
+void
+threadlist_view_update_thread_visual(ThreadlistView *view,
+				     OchushaBBSThread *thread,
+				     ThreadlistEntryVisualParameter *parameter)
+{
+  gpointer data = NULL;
+  ThreadlistInfo *info;
+  g_return_if_fail(IS_THREADLIST_VIEW(view) && thread != NULL);
+
+  info = ensure_threadlist_info(thread);
+
+  if (info->threadlist_view != view)
+    {
+#if DEBUG_WIDGET
+      fprintf(stderr, "info->threadlist_view=0x%x, view=0x%x\n",
+	      (int)info->threadlist_view, (int)view);
+#endif
+      return;
+    }
+
+  if (!gtk_list_store_iter_is_valid(GTK_LIST_STORE(view->model), &info->iter))
+    {
+      /* 줿ե륿ξ֤ѤäơiterŤʤäȤ롣 */
+      return;
+    }
+
+  gtk_tree_model_get(view->model, &info->iter,
+		     THREADLIST_DATA, &data, -1);
+  g_return_if_fail(thread == data);
+
+  gtk_list_store_set(GTK_LIST_STORE(view->model), &info->iter,
+		     THREADLIST_VISUAL_PARAMETER,
+		     threadlist_view_encode_visual_params(view, parameter),
+		     -1);
+}
+
+
 static void
 decorate_threadlist_entry(GtkTreeViewColumn *tree_column,
 			  GtkCellRenderer *renderer, GtkTreeModel *tree_model,
-			  GtkTreeIter *iter, gpointer unused)
+			  GtkTreeIter *iter, gpointer view)
 {
-  GValue flags = { 0, };
+  guint flags;
+  guint color_code;
+  GValue fg = { 0, };
   GValue bg = { 0, };
 
   g_return_if_fail(gtk_list_store_iter_is_valid(GTK_LIST_STORE(tree_model),
 						iter));
-  gtk_tree_model_get_value(tree_model, iter, THREADLIST_MARK, &flags);
-  g_value_init(&bg, G_TYPE_STRING);
-  if (g_value_get_int(&flags) & BBS_THREAD_HIGHLIGHT)
-    g_value_set_static_string(&bg, "PowderBlue");
-  else
-    g_value_set_static_string(&bg, NULL);
+  gtk_tree_model_get(tree_model, iter,
+		     THREADLIST_VISUAL_PARAMETER, &flags, -1);
 
+  if (!GTK_IS_CELL_RENDERER_TOGGLE(renderer))
+    {
+      color_code
+	= threadlist_view_decode_visual_param_foreground_color(view, flags);
+      g_value_init(&fg, G_TYPE_STRING);
+      g_value_set_static_string(&fg, list_entry_foreground_color[color_code]);
+      g_object_set_property((GObject *)renderer, "foreground", &fg);
+    }
+
+  color_code =
+    threadlist_view_decode_visual_param_background_color(view, flags);
+  g_value_init(&bg, G_TYPE_STRING);
+  g_value_set_static_string(&bg, list_entry_background_color[color_code]);
   g_object_set_property((GObject *)renderer, "cell_background", &bg);
 
   if (!GTK_IS_CELL_RENDERER_TEXT(renderer))
     return;
 
-  if (g_value_get_int(&flags) & BBS_THREAD_HIDDEN)
+  if (threadlist_view_decode_visual_param_flags(view, flags)
+      & THREADLIST_ENTRY_FLAG_STRIKE)
     {
       GValue st = { 0, };
       g_value_init(&st, G_TYPE_BOOLEAN);
@@ -1134,18 +1603,21 @@ decorate_threadlist_entry(GtkTreeViewColumn *tree_column,
 static void
 active_set_cell_data_flags(GtkTreeViewColumn *tree_column,
 			   GtkCellRenderer *renderer, GtkTreeModel *tree_model,
-			   GtkTreeIter *iter, gpointer unused)
+			   GtkTreeIter *iter, gpointer view)
 {
-  GValue flags = { 0, };
+  guint flags;
   GtkCellRendererToggle *toggle_renderer = GTK_CELL_RENDERER_TOGGLE(renderer);
   g_return_if_fail(gtk_list_store_iter_is_valid(GTK_LIST_STORE(tree_model),
 						iter));
 
-  gtk_tree_model_get_value(tree_model, iter, THREADLIST_MARK, &flags);
+  gtk_tree_model_get(tree_model, iter,
+		     THREADLIST_VISUAL_PARAMETER, &flags, -1);
+
+  flags = threadlist_view_decode_visual_param_flags(view, flags);
 
   gtk_cell_renderer_toggle_set_active(toggle_renderer,
-				g_value_get_int(&flags) & BBS_THREAD_FAVORITE);
-  decorate_threadlist_entry(tree_column, renderer, tree_model, iter, unused);
+				      flags & THREADLIST_ENTRY_FLAG_MARK);
+  decorate_threadlist_entry(tree_column, renderer, tree_model, iter, view);
 }
 
 
@@ -1153,7 +1625,7 @@ static void
 text_set_cell_data_rank_diff(GtkTreeViewColumn *tree_column,
 			     GtkCellRenderer *renderer,
 			     GtkTreeModel *tree_model, GtkTreeIter *iter,
-			     gpointer unused)
+			     gpointer view)
 {
   GValue src = { 0, };
   GValue value = { 0, };
@@ -1179,7 +1651,7 @@ text_set_cell_data_rank_diff(GtkTreeViewColumn *tree_column,
   else if (diff == RANK_SAME_RANK_DOWN_MAGIC)
     {
       /* ľΥȥƱ̥ξ */
-      g_value_set_static_string(&value, _("Down"));
+      g_value_set_static_string(&value, _("down"));
     }
   else if (diff > 0)
     {
@@ -1195,12 +1667,12 @@ text_set_cell_data_rank_diff(GtkTreeViewColumn *tree_column,
     }
 
   g_object_set_property((GObject *)renderer, "text", &value);
-  decorate_threadlist_entry(tree_column, renderer, tree_model, iter, unused);
+  decorate_threadlist_entry(tree_column, renderer, tree_model, iter, view);
 }
 
 
 gboolean
-threadlist_view_open(ThreadlistView *view, ThreadlistInfoNewFunc *factory)
+threadlist_view_open(ThreadlistView *view)
 {
   GtkListStore *store;
 
@@ -1221,9 +1693,7 @@ threadlist_view_open(ThreadlistView *view, ThreadlistInfoNewFunc *factory)
 
   gtk_tree_view_set_model(view->view, GTK_TREE_MODEL(store));
   view->model = GTK_TREE_MODEL(store);
-  view->threadlist_info_new = factory;
 
-  view->number_of_threads = 0;
   view->last_rank_diff = 0;
 
   return TRUE;
@@ -1231,34 +1701,38 @@ threadlist_view_open(ThreadlistView *view, ThreadlistInfoNewFunc *factory)
 
 
 void
-threadlist_view_close(ThreadlistView *view)
+threadlist_view_close(ThreadlistView *view, OchushaBBSThread *last_thread)
 {
+  ThreadlistInfo *info;
   g_return_if_fail(IS_THREADLIST_VIEW(view));
+
   view->model_is_opened = FALSE;
+  info = ensure_threadlist_info(last_thread);
+  g_return_if_fail(info != NULL
+		   && gtk_list_store_iter_is_valid(GTK_LIST_STORE(view->model),
+						   &info->iter));
+  if (view->last_path != NULL)
+    gtk_tree_path_free(view->last_path);
+  view->last_path = gtk_tree_model_get_path(view->model, &info->iter);
 }
 
 
 void
 threadlist_view_append_thread(ThreadlistView *view, OchushaBBSThread *thread,
-			      gint rank)
+			      gint last_rank, gint new_rank)
 {
-  ThreadlistInfo *info = thread->user_data;
-
+  ThreadlistInfo *info;
   g_return_if_fail(IS_THREADLIST_VIEW(view));
-  g_assert(view->model != NULL);
+  g_return_if_fail(OCHUSHA_IS_BBS_THREAD(thread));
 
-  if (info == NULL)
-    {
-      if (view->threadlist_info_new != NULL)
-	info = (*view->threadlist_info_new)(thread);
-      else
-	info = (ThreadlistInfo *)calloc(1, sizeof(ThreadlistInfo));
-      thread->user_data = info;
-    }
+  info = ensure_threadlist_info(thread);
+
+  g_assert(view->model != NULL);
 
   gtk_list_store_append(GTK_LIST_STORE(view->model), &info->iter);
   info->threadlist_view = view;
-  set_thread_values(view, GTK_LIST_STORE(view->model), rank, thread);
+  set_thread_values(view, GTK_LIST_STORE(view->model),
+		    last_rank, new_rank, thread);
 }
 
 
@@ -1267,14 +1741,14 @@ threadlist_view_update_thread(ThreadlistView *view, OchushaBBSThread *thread)
 {
   gpointer data = NULL;
   ThreadlistInfo *info;
-  g_return_if_fail(IS_THREADLIST_VIEW(view) && thread != NULL
-		   && thread->user_data != NULL);
+  g_return_if_fail(IS_THREADLIST_VIEW(view));
+  g_return_if_fail(OCHUSHA_IS_BBS_THREAD(thread));
 
-  info = (ThreadlistInfo *)thread->user_data;
+  info = ensure_threadlist_info(thread);
 
   if (info->threadlist_view != view)
     {
-#if DEBUG_WIDGET
+#if DEBUG_WIDGET	/* ΥƥȼΤĹ */
       fprintf(stderr, "info->threadlist_view=0x%x, view=0x%x\n",
 	      (int)info->threadlist_view, (int)view);
 #endif
@@ -1296,7 +1770,7 @@ threadlist_view_update_thread(ThreadlistView *view, OchushaBBSThread *thread)
 #endif
   g_return_if_fail(thread == data);
 
-  set_thread_values(view, GTK_LIST_STORE(view->model), RANK_UNSPECIFIED,
+  set_thread_values(view, GTK_LIST_STORE(view->model), 0, RANK_UNSPECIFIED,
 		    thread);
 }
 
@@ -1308,10 +1782,10 @@ threadlist_view_scroll_to_thread(ThreadlistView *view,
   gpointer data = NULL;
   GtkTreePath *path;
   ThreadlistInfo *info;
-  g_return_if_fail(IS_THREADLIST_VIEW(view) && thread != NULL);
+  g_return_if_fail(IS_THREADLIST_VIEW(view));
+  g_return_if_fail(OCHUSHA_IS_BBS_THREAD(thread));
 
-  info = (ThreadlistInfo *)thread->user_data;
-  g_return_if_fail(info != NULL);
+  info = ensure_threadlist_info(thread);
 
   if (info->threadlist_view != view
       || !gtk_list_store_iter_is_valid(GTK_LIST_STORE(view->model),
