@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ugly_gtk2utils.c,v 1.1.1.1 2003/05/10 16:34:26 fuyu Exp $
+ * $Id: ugly_gtk2utils.c,v 1.3 2003/05/14 08:44:24 fuyu Exp $
  */
 
 #include "ugly_gtk2utils.h"
@@ -43,6 +43,12 @@ ugly_gtk_tree_view_unbind_keys(void)
 
   gtk_binding_entry_remove(binding_set, GDK_space, 0);
   gtk_binding_entry_remove(binding_set, GDK_BackSpace, 0);
+  gtk_binding_entry_remove(binding_set, GDK_f, GDK_CONTROL_MASK);
+  gtk_binding_entry_remove(binding_set, GDK_F, GDK_CONTROL_MASK);
+  gtk_binding_entry_remove(binding_set, GDK_Up, 0);
+  gtk_binding_entry_remove(binding_set, GDK_Down, 0);
+  gtk_binding_entry_remove(binding_set, GDK_p, GDK_CONTROL_MASK);
+  gtk_binding_entry_remove(binding_set, GDK_n, GDK_CONTROL_MASK);
 
   g_type_class_unref(klass);
 
