@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_category.c,v 1.1.1.1 2003/05/10 16:34:28 fuyu Exp $
+ * $Id: ochusha_board_category.c,v 1.2 2003/05/14 17:14:19 fuyu Exp $
  */
 
 
@@ -242,7 +242,7 @@ ochusha_board_category_set_name(OchushaBoardCategory *category,
   if (category->name != NULL)
     g_free(category->name);
 
-  category->name = g_strdup(name);
+  category->name = wipe_string(name);
 }
 
 
