@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.c,v 1.1.1.1 2003/05/10 16:34:28 fuyu Exp $
+ * $Id: ochusha_bulletin_board.c,v 1.3 2003/05/14 20:58:12 fuyu Exp $
  */
 
 
@@ -210,8 +210,6 @@ ochusha_bulletin_board_init(OchushaBulletinBoard *board)
   board->dropped_list = NULL;
 
   board->bbs_type = OCHUSHA_BBS_TYPE_UNKNOWN;
-
-  board->user_data = NULL;
 }
 
 
@@ -336,7 +334,7 @@ ochusha_bulletin_board_set_name(OchushaBulletinBoard *board,
   if (board->name != NULL)
     g_free(board->name);
 
-  board->name = g_strdup(name);
+  board->name = wipe_string(name);
 }
 
 
