@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.h,v 1.1.1.1 2003/05/10 16:34:28 fuyu Exp $
+ * $Id: ochusha_bulletin_board.h,v 1.2 2003/05/14 20:58:12 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BULLETIN_BOARD_H_
@@ -61,7 +61,7 @@ typedef enum
 {
   OCHUSHA_BBS_TYPE_UNKNOWN = -1,
   OCHUSHA_BBS_TYPE_2CH = 0,
-  OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP
+  OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP	/* Ϥ¸ߤƤʤ */
 } OchushaBBSType;
 
 
@@ -82,8 +82,6 @@ struct _OchushaBulletinBoard
   GSList *dropped_list;		/* DAT(OchushaBBSThread *)ꥹ */
 
   OchushaBBSType bbs_type;	/* ǼĤμࡽ2chΤߤ */
-
-  gpointer user_data;
 };
 
 
