@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: utils.c,v 1.1.1.1 2003/05/10 16:34:28 fuyu Exp $
+ * $Id: utils.c,v 1.2 2003/05/14 17:14:19 fuyu Exp $
  */
 
 #define OCHUSHA_UTILS_IMPLEMENTATION
@@ -207,141 +207,34 @@ convert_string(iconv_t converter, const char *src, int length)
 }
 
 
-#if 1	/* to be obsoleted soon */
 /*
- * url_extract_http_server
- *
- * Ϳ줿ʸhttpФURLǤȤƥ̾ʬ֤
- * ˼ԤˤNULL֤
+ * wipe_string:
+ *   string˴ޤޤnon-printableʸȥĤζʸ
+ *   ԡ֤
  */
-char *
-url_extract_http_server(const char *url)
+gchar *
+wipe_string(const gchar *string)
 {
-  /*
-   * ݤʤΤǡurlhttp://server/.*ʷǤȷǤġ
-   */
-  char *cur_pos = strstr(url, "http://");
-  char *hostpart_tail;
-  int length;
-  char buffer[MAX_HOSTNAME_LEN];
+  gchar *result_string = g_strchomp(g_strdup(string));
+  gchar *head = result_string;
 
-  if (cur_pos == NULL)
+  while (*head != '\0')
     {
-#if DEBUG_UTILS
-      fprintf(stderr, "Not a http URL: %s\n", url);
-#endif
-      return NULL;
-    }
-  cur_pos += 7;	/* skip "http://" */
-
-  hostpart_tail = strchr(cur_pos, '/');
-  if (hostpart_tail == NULL)
-    {
-      if (*cur_pos != '\n')
-	return strdup(cur_pos);
-      else
-	return NULL;
-    }
-
-  length = hostpart_tail - cur_pos;
-  if (length <= 0 || length >= MAX_HOSTNAME_LEN)
-    {
-#if DEBUG_UTILS
-      fprintf(stderr, "Couldn't parse URL: %s\n", url);
-#endif
-      return NULL;
-    }
-
-  memcpy(buffer, cur_pos, length);
-  buffer[length] = '\0';
-  return strdup(buffer);
-}
-
-
-/*
- * url_extract_http_root_pathname
- *
- * Ϳ줿ʸhttpФURLǤȤƥɥȤΥǥ쥯ȥ
- * ΥФΥ롼ȤХѥ֤̾
- * ex) http://pc.2ch.net/unix/  unix/
- * ˼ԤˤNULL֤
- */
-char *
-url_extract_http_root_pathname(const char *url)
-{
-  /*
-   * ݤʤΤǡurlhttp://server/.*ʷǤȷǤġ
-   */
-  char *cur_pos = strstr(url, "http://");
-
-  if (cur_pos == NULL)
-    {
-#if DEBUG_UTILS
-      fprintf(stderr, "Couldn't parse URL: %s\n", url);
-#endif
-      return NULL;
-    }
-  cur_pos += 7; /* skip "http://" */
-  cur_pos = strchr(cur_pos, '/');
-  if (cur_pos == NULL)
-    {
-#if DEBUG_UTILS
-      fprintf(stderr, "Couldn't get path parts: %s\n", url);
-#endif
-      return NULL;
-    }
-
-  return strdup(cur_pos + 1);
-}
-
-
-char *
-url_encode_string(const char *src)
-{
-  int src_len = strlen(src);
-  char *buffer = (char *)malloc(src_len * 3 + 1);
-  const unsigned char *cur_src = src;
-  char *cur_dst = buffer;
-  while (*cur_src != '\0')
-    {
-      sprintf(cur_dst, "%%%02X", *cur_src++);
-      cur_dst += 3;
-    }
-  *cur_dst = '\0';
-
-  return buffer;
-}
-
-
-char *
-url_decode_string(const char *src)
-{
-  int src_len = strlen(src);
-  char *buffer = (char *)malloc(src_len + 1);
-  const char *cur_src = src;
-  char *cur_dst = buffer;
-
-  while (*cur_src != '\0')
-    {
-      if (*cur_src != '%')
-	*cur_dst++ = *cur_src++;
-      else
-	{
-	  unsigned int code;
-	  int result = sscanf(cur_src, "%%%02x", &code);
-	  if (result == 1)
-	    {
-	      *cur_dst++ = (char)code;
-	      cur_src += 3;
-	    }
-	  else
-	    *cur_dst++ = *cur_src++;
+      if (*head > '\0' && *head <= '\37')
+	{	/* ȯˤĤʬФ */
+	  int len;
+	  gchar *tmp_pos;
+	  for (tmp_pos = g_utf8_next_char(head);
+	       *tmp_pos > '\0' && *tmp_pos <= '\37';
+	       tmp_pos = g_utf8_next_char(tmp_pos));
+	  len = strlen(tmp_pos);
+	  memmove(head, tmp_pos, len + 1);
 	}
+      else
+	head = g_utf8_next_char(head);
     }
-  *cur_dst = '\0';
-  return buffer;
+  return result_string;
 }
-#endif
 
 
 /*
